/*
Created by: X
http://www.createdbyx.com/

xGameConsoleXNA is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/

This code was converted from vb.net to C#.
Original vb.net compact framework code can be found here
http://www.danielmoth.com/Blog/2004/12/backgroundworker-for-cf-10.html
http://www.danielmoth.com/Blog/BackgroundWorker.vb
 
Special thanks to Daniel Moth http://www.danielmoth.com/Blog/ for making the original vb.net code availible. 
 */

//1/12/2004 - Daniel Moth creates BackgroundWorker (http://www.danielmoth.com/Blog/) 
//4/12/2004 - DDGM:	Changed text in comments, and exception messages. 
//                    Made RunWorkerAsync throw if no eventhandler attached
//                    Added ctor to make the class work on the full framework as is
//
//8/6/2007  - Dean Lunz (aka Created by: X) Converted to C# for use with XNA so 
//            the code could be run on both windows and the xbox360 

using System;
using System.Collections.Generic;
using System.Text;

namespace xGameConsoleXNA
{


    #region EventArgs classes
    public class RunWorkerCompletedEventArgs : EventArgs
    {

        // This class should inherit from AsyncCompletedEventArgs but I don't see the point in the CF's case

        private object mResult;
        private bool mCancelled;
        private Exception mError;

        public RunWorkerCompletedEventArgs(object aResult, Exception aError, bool aCancelled)
        {
            mResult = aResult;
            mError = aError;
            mCancelled = aCancelled;
        }

        public object Result
        {
            get { return mResult; }
        }

        public bool Cancelled
        {
            get { return mCancelled; }
        }

        public Exception Error
        {
            get { return mError; }
        }
    }

    public class ProgressChangedEventArgs : EventArgs
    {
        private int mProgressPercent;
        private object mUserState;

        public ProgressChangedEventArgs(int aProgressPercent, object aUserState)
        {
            mProgressPercent = aProgressPercent;
            mUserState = aUserState;
        }

        public int ProgressPercentage
        {
            get { return mProgressPercent; }
        }

        public object UserState
        {
            get { return mUserState; }
        }
    }

    public class DoWorkEventArgs : System.ComponentModel.CancelEventArgs
    {

        private object mArgument;
        private object mResult;

        public DoWorkEventArgs(object aArgument)
        {
            mArgument = aArgument;
        }

        public object Argument
        {
            get { return mArgument; }
        }

        public object Result
        {
            get { return mResult; }
            set { mResult = value; }
        }
    }
    #endregion

    #region Delegates for 3 events of class
    /// <summary>
    /// Provides a delegate used for handling DoWork events.
    /// </summary>
    /// <param name="sender">The object that raised the event.</param>
    /// <param name="e">A reference to a <see cref="DoWorkEventArgs"/> object.</param>
    public delegate void DoWorkEventHandler(object sender, DoWorkEventArgs e);
    /// <summary>
    /// Provides a delegate used for handling ProgressChanged events.
    /// </summary>
    /// <param name="sender">The object that raised the event.</param>
    /// <param name="e">A reference to a <see cref="ProgressChangedEventArgs"/> object.</param>
    public delegate void ProgressChangedEventHandler(object sender, ProgressChangedEventArgs e);
    /// <summary>
    /// Provides a delegate used for handling WorkerCompleted events.
    /// </summary>
    /// <param name="sender">The object that raised the event.</param>
    /// <param name="e">A reference to a <see cref="RunWorkerCompletedEventArgs"/> object.</param>
    public delegate void RunWorkerCompletedEventHandler(object sender, RunWorkerCompletedEventArgs e);
    #endregion

    #region BackgroundWorker Class
    ///<summary>
    ///Executes an operation on a separate thread.
    ///</summary>
    public class BackgroundWorker : System.ComponentModel.Component
    {


        #region Public Interface
        /// <summary>
        /// Occurs when RunWorkerAsync is called.
        /// </summary>
        public event DoWorkEventHandler DoWork;
        /// <summary>
        /// Occurs when ReportProgress is called.
        /// </summary>
        public event ProgressChangedEventHandler ProgressChanged;
        /// <summary>
        /// Occurs when the background operation has completed, has been canceled, or has raised an exception.
        /// </summary>
        public event RunWorkerCompletedEventHandler RunWorkerCompleted;

        /// <summary>
        /// Initializes a new instance of the BackgroundWorker class.
        /// </summary>
        public BackgroundWorker()
        {
            //Me.New(New System.Windows.Forms.Control)
            // ideally we want to call Control.CreateControl()
            // without it, running on the desktop will crash (it is OK on the CF)
            // [on the full fx simply calling a control's constructor does not create the Handle.]

            // The CreateControl method is not supported on the CF so to keep this assembly retargettable
            // I have offered the alternative ctor for desktop clients 
            // (where they can pass in already created controls)
        }

        ///// <summary>
        ///// Initializes a new instance of the BackgroundWorker class.
        ///// Call from the desktop code as the other ctor is not good enough
        ///// Call it passing in a created control e.g. the Form
        ///// </summary>
        //public BackgroundWorker(ByVal aControl As System.Windows.Forms.Control)
        //    MyBase.New()
        //    mGuiMarshaller = aControl
        //End Sub

        /// <summary>
        /// Gets a value indicating whether the application has requested cancellation of a background operation.
        /// </summary>
        public bool CancellationPending
        {
            get { return mCancelPending; }
        }

        /// <summary>
        /// Raises the BackgroundWorker.ProgressChanged event.
        /// </summary>
        /// <param name="aProgressPercent">The percentage, from 0 to 100, of the background operation that is complete. </param>
        public void ReportProgress(int aProgressPercent)
        {
            this.ReportProgress(aProgressPercent, null);
        }

        /// <summary>
        /// Raises the BackgroundWorker.ProgressChanged event.
        /// </summary>
        /// <param name="aProgressPercent">The percentage, from 0 to 100, of the background operation that is complete. </param>
        /// <param name="aUserState">The state object passed to BackgroundWorker.RunWorkerAsync(System.Object).</param>
        public void ReportProgress(int aProgressPercent, object aUserState)
        {
            if (!mDoesProgress)
            {
                throw new System.InvalidOperationException("Doesn't do progress events. You must WorkerReportsProgress=True");
            }

            //Send the event to the GUI
            System.Threading.ThreadPool.QueueUserWorkItem(
            new System.Threading.WaitCallback(this.ProgressHelper),
            new ProgressChangedEventArgs(aProgressPercent, aUserState));
        }

        /// <summary>
        /// Starts execution of a background operation.
        /// </summary>
        public void RunWorkerAsync()
        {
            this.RunWorkerAsync(null);
        }
                
        /// <summary>
        /// Starts execution of a background operation.
        /// </summary>
        /// <param name="aArgument"> A parameter for use by the background operation to be executed in the BackgroundWorker.DoWork event handler.</param>
        public void RunWorkerAsync(object aArgument)
        {
            if (mInUse)
            {
                throw new System.InvalidOperationException("Already in use.");
            }

            if (this.DoWork == null)
            {
                throw new System.InvalidOperationException("You must subscribe to the DoWork event.");
            }

            mInUse = true;
            mCancelPending = false;

            System.Threading.ThreadPool.QueueUserWorkItem(
            new System.Threading.WaitCallback(this.DoTheRealWork), aArgument);
        }

        /// <summary>
        /// Requests cancellation of a pending background operation.
        /// </summary>
        public void CancelAsync()
        {
            if (!mDoesCancel)
            {
                throw new System.InvalidOperationException("Does not support cancel. You must WorkerSupportsCancellation=True");
            }

            mCancelPending = true;
        }

        /// <summary>
        /// Gets or sets a value indicating whether the BackgroundWorker object can report progress updates.
        /// </summary>
        public bool WorkerReportsProgress
        {
            get { return mDoesProgress; }
            set { mDoesProgress = value; }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the BackgroundWorker object supports asynchronous cancellation.
        /// </summary>
        public bool WorkerSupportsCancellation
        {
            get { return mDoesCancel; }
            set { mDoesCancel = value; }
        }
        #endregion

        #region Fields
        //Ensures the component is used only once per session
        private bool mInUse;

        //Stores the cancelation request that the worker thread (user's code) should check via CancellationPending
        private bool mCancelPending;

        //Whether the object supports cancelling or not (and progress or not)
        private bool mDoesCancel;
        private bool mDoesProgress;

        //Helper objects since Control.Invoke takes no arguments
        private RunWorkerCompletedEventArgs mFinalResult;
        private ProgressChangedEventArgs mProgressArgs;

        // Helper for marshalling execution to GUI thread
        //private mGuiMarshaller As System.Windows.Forms.Control
        #endregion

        #region Private Methods
        // Async(ThreadPool) called by ReportProgress for reporting progress
        private void ProgressHelper(object o)
        {
            mProgressArgs = (ProgressChangedEventArgs)o;
            // mGuiMarshaller.Invoke(new System.EventHandler(this.TellThemOnGuiProgress));
        }

        //// ControlInvoked by ProgressHelper for raising progress
        //private void TellThemOnGuiProgress(object sender, EventArgs e)
        //{
        //    if (this.ProgressChanged != null) ProgressChanged(this, mProgressArgs);
        //}

        // Async(ThreadPool) called by RunWorkerAsync [the little engine of this class]
        private void DoTheRealWork(object o)
        {
            // declare the vars we will pass back to client on completion
            Exception er = null;
            bool ca = false;
            object result = null;

            // Raise the event passing the original argument and catching any exceptions
            try
            {
                DoWorkEventArgs inOut = new DoWorkEventArgs(o);
                if (this.DoWork != null) DoWork(this, inOut);
                ca = inOut.Cancel;
                result = inOut.Result;
            }
            catch (Exception ex)
            {
                er = ex;
            }

            // store the completed final result in a temp var
            RunWorkerCompletedEventArgs tempResult = new RunWorkerCompletedEventArgs(result, er, ca);

            // return execution to client by going async here
            System.Threading.ThreadPool.QueueUserWorkItem(
             new System.Threading.WaitCallback(this.RealWorkHelper), tempResult);

            // prepare for next use
            mInUse = false;
            mCancelPending = false;
        }

        // Async(ThreadPool) called by DoTheRealWork [to avoid any rentrancy issues at the client end]
        private void RealWorkHelper(object o)
        {
            mFinalResult = (RunWorkerCompletedEventArgs)o;
            // mGuiMarshaller.Invoke(new System.EventHandler(this.TellThemOnGuiCompleted));
        }

        //// ControlInvoked by RealWorkHelper for raising final completed event
        //private void TellThemOnGuiCompleted(object sender, EventArgs e)
        //{
        //    if (this.RunWorkerCompleted != null) RunWorkerCompleted(this, mFinalResult);
        //}
        #endregion
    }
    #endregion


}
