/*
Created by: X
http://www.createdbyx.com/

xGameConsoleXNA is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
using System.Diagnostics;
//using System.Data;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System;

namespace xGameConsoleXNA
{
    /// <summary>
    /// Provides a base foundation that makes it very easy to add console comamnds.
    /// </summary>
    public abstract class BaseCommands : ICommandExtender, IDisposable
    {
        
        internal CommandManager mobjManager;
        internal CommandExtenderManager mobjExtenderManager;
        /// <summary>
        /// Set to true if the OnConnect method completed succesfully and set to false when OnDisconnet is called.
        /// </summary>
       protected internal bool mblnIsConnected = false;
        internal StringCollection mobjCommandList;
        internal bool mblnRemovingCommands = false;
        internal Console mobjConsoleReference;


        /// <summary>
        /// Returns a reference to the extender manager object.
        /// </summary>
        public CommandExtenderManager CommandExtenderManager
        {
            get { return mobjExtenderManager; }
        }

        /// <summary>
        /// Returns a reference to the command manager object.
        /// </summary>
        public CommandManager CommandManager
        {
            get { return mobjManager; }
        }

        /// <summary>
        /// Returns a reference to the console.
        /// </summary>
        public xGameConsoleXNA.Console ConsoleReference
        {
            get { return mobjConsoleReference; }
        }

        /// <summary>
        /// Adds a new command to the console.
        /// </summary>
        /// <param name="Name">The name of the command to be added.</param>
        /// <param name="callback">A delegate that will be called when the command is executed on the console.</param>
        protected internal void DoAddCommand(string Name, CommandCallback callback)
        {
            if (disposed) return;
            if (mobjManager == null) return;
            if (mobjExtenderManager.CommandManager == null) return;

            mobjManager.AddCommand(Name, callback);
            mobjCommandList.Add(Name);
        }

        internal void CommandIsBeingRemoved(string CommandName, System.ComponentModel.CancelEventArgs e)
        {
            if (disposed) return;

            // we can now prevent any of our commands from being removed
            e.Cancel = (mobjCommandList.IndexOf(CommandName) != -1) && (mblnRemovingCommands == false);
            if (e.Cancel)
            {
                if (mblnIsConnected == false) return;

                if (mobjExtenderManager.CommandManager == null) return;

                this.Print("Some process tried to remove command " + CommandName);
            }
        }

        /// <summary>
        /// Handles the CommandExecuted event.
        /// </summary>
        /// <param name="Sender">The object that sent the event.</param>
        /// <param name="e">A ReturnValueEventArgs object containing the return value and information about the command that was executed.</param>
        protected virtual void CommandEvents_CommandExecuted(object Sender, ReturnValueEventArgs e)
        {
        }

        internal void DoRemoveCommands()
        {
            if (disposed) return;

            if (mobjManager == null) return;

            if (mobjExtenderManager.CommandManager == null) return;

            mblnRemovingCommands = true;
            foreach (string tmp in mobjCommandList)
            {
                mobjManager.RemoveCommand(tmp);
            }
            mobjCommandList.Clear();
            mblnRemovingCommands = false;
        }

        /// <summary>
        /// Prints a blank line in the console.
        /// </summary>
        public void PrintBlankLine()
        {
            mobjConsoleReference.PrintBlankLine();
        }

        /// <summary>
        /// Prints text out to the console.
        /// </summary>
        /// <param name="Text">The text to be printed.</param>
        public void Print(string Text)
        {
            mobjConsoleReference.Print(Text);
        }

        /// <summary>
        /// Prints an array of text out to the console.
        /// </summary>
        /// <param name="Text">The array of text to be printed.</param>
        public void Print(string[] Text)
        {
            mobjConsoleReference.Print(Text);
        }

        /// <summary>
        /// Prints an collection of text out to the console.
        /// </summary>
        /// <param name="Text">The collection of text to be printed.</param>
        public void Print(StringCollection Text)
        {
            mobjConsoleReference.Print(Text);
        }

        /// <summary>
        /// Returns true if this object has been connected to the extender manager.
        /// </summary>
        public virtual bool IsConnected
        {
            get { return mblnIsConnected; }
        }

        /// <summary>
        /// Connects the object to the extender manager.
        /// </summary>
        /// <param name="Manager">A reference to a CommandManager object.</param>
        /// <param name="ExtenderManager">A reference to a CommandExtenderManager object.</param>
        /// <remarks>Override this method and add your own commands to the command manager. Commands that are added
        /// to the command manager will automatically be removed when a call to the OnDisconnect method is made. 
        /// Additionally this object will also try to prevent the comamnds that it has added to the console from being 
        /// removed by other objects by hooking into the BeforeRemoveCommand event of the command manager.</remarks>
        /// <example>An example of how to override the method.
        /// <code>
        ///  public override void OnConnect(xGameConsoleXNA.CommandManager Manager, xGameConsoleXNA.CommandExtenderManager ExtenderManager)
        ///  {
        ///      // be sure to call the base OnConnect method
        ///      base.OnConnect(Manager, ExtenderManager);
        ///      // add a quit command
        ///      DoAddCommand("quit", new CommandCallback(this.DoQuit));
        ///  }
        /// </code>
        /// </example>
        /// <exception cref="ArgumentNullException">Is thrown if either the Manager or ExtenderManager parameters is null (Nothing in vb.net).</exception>
        public virtual void OnConnect(CommandManager Manager, CommandExtenderManager ExtenderManager)
        {
            // if already disposed we can exit
            if (disposed) return;
            if (Manager == null) throw new ArgumentNullException("Manager");
            if (ExtenderManager == null) throw new ArgumentNullException("ExtenderManager");

            mobjManager = Manager;
            mobjManager.AfterCommandExecuted += new xGameConsoleXNA.CommandManager.AfterCommandExecutedEventHandler(CommandEvents_CommandExecuted);
            mobjExtenderManager = ExtenderManager;
            if (mobjManager == null) return;

            if (mobjExtenderManager.CommandManager == null) return;

            mobjCommandList = new StringCollection();
            mobjManager.BeforeRemoveCommand += new xGameConsoleXNA.CommandManager.BeforeRemoveCommandEventHandler(CommandIsBeingRemoved);
            mblnIsConnected = true;
        }

        /// <summary>
        /// Disconnects the object from the extender manager.
        /// </summary>
        /// <remarks>Removes any commands from the command manager if commands were previously added
        /// during the call to the OnConnect method.</remarks>
        public virtual void OnDisconnect()
        {
            // if already disposed we can exit
            if (disposed) return;
            // if already disconnected just exit
            if (mblnIsConnected == false) return;

            // clear the commands that we have added to the console
            mblnIsConnected = false;
            mobjManager.BeforeRemoveCommand -= new xGameConsoleXNA.CommandManager.BeforeRemoveCommandEventHandler(CommandIsBeingRemoved);
            mobjManager.AfterCommandExecuted -= new xGameConsoleXNA.CommandManager.AfterCommandExecutedEventHandler(CommandEvents_CommandExecuted);
            // remove any commands that this object has added to the console
            DoRemoveCommands();
            // release referencs to varibles
            mobjCommandList = null;
            mobjManager = null;
        }


        #region " IDisposable Support "
        private bool disposed = false;
        // IDisposable
        private void Dispose(bool disposing)
        {
            if (!this.disposed)
            {
                if (disposing)
                {
                    DoRemoveCommands();
                    if (mobjCommandList != null) mobjCommandList.Clear();
                    mobjCommandList = null;

                    mobjManager.BeforeRemoveCommand -= new xGameConsoleXNA.CommandManager.BeforeRemoveCommandEventHandler(CommandIsBeingRemoved);
                    mobjManager.AfterCommandExecuted -= new xGameConsoleXNA.CommandManager.AfterCommandExecutedEventHandler(CommandEvents_CommandExecuted);

                    mobjManager = null;
                }
                // put code to free unmanaged resources here
            }
            this.disposed = true;
        }

        // This code added by Visual Basic to correctly implement the disposable pattern.
        public virtual void Dispose()
        {
            // Do not change this code.  Put cleanup code in Dispose(ByVal disposing As Boolean) above.
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        ~BaseCommands()
        {
            // Do not change this code.  Put cleanup code in Dispose(ByVal disposing As Boolean) above.
            Dispose(false);
            // ~base();
        }
        #endregion

        /// <summary>
        /// Initializes a new instance of the BaseCommands class.
        /// </summary>
        /// <param name="Console">A reference to a Console object.</param>
        /// <exception cref="ArgumentNullException">Is thrown if the Console parameter is null (Nothing in vb.net).</exception>
        public BaseCommands(xGameConsoleXNA.Console Console)
        {
            if (Console == null) throw new ArgumentNullException("Console");
            this.mobjConsoleReference = Console;
        }
    }
}
