/*
Created by: X
http://www.createdbyx.com/

xGameConsoleXNA is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
// Special thanks to Jason Olson from which this code is originally based on.
// http://geekswithblogs.net/jolson/
// http://geekswithblogs.net/jolson/archive/2004/02/16/2174.aspx

using System.Diagnostics;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System;
using xGameConsoleXNA.Properties;

namespace xGameConsoleXNA
{

    /// <summary>
    /// Manages commands for the console.
    /// </summary>
    public class CommandManager : IDisposable
    {
        // Inner Types
        private class Command
        {
            public string Name;
            public CommandCallback Callback;
            public string[] Arguments;
        }

        // strings that define the beggining and end of function calls
        /// <summary>
        /// Gets/Sets the string for declaring the begining of a function call.
        /// </summary>
        protected string startFunction = "[";
        /// <summary>
        /// Gets/Sets the string for declaring the end of a function call.
        /// </summary>
        protected string endFunction = "]";


        /// <summary>
        /// Gets/Sets the string for declaring the begining of a function call.
        /// </summary>
        public string StartFunction
        {
            get { return startFunction; }
            set { startFunction = value; }
        }

        /// <summary>
        /// Gets/Sets the string for declaring the end of a function call.
        /// </summary>
        public string EndFunction
        {
            get { return endFunction; }
            set { endFunction = value; }
        }

        #region Event declorations ...
        public delegate void BeforeCommandExecutedEventHandler(object Sender, ReturnValueEventArgs e);
        private BeforeCommandExecutedEventHandler BeforeCommandExecutedEvent;

        public event BeforeCommandExecutedEventHandler BeforeCommandExecuted
        {
            add
            {
                BeforeCommandExecutedEvent = (BeforeCommandExecutedEventHandler)System.Delegate.Combine(BeforeCommandExecutedEvent, value);
            }
            remove
            {
                BeforeCommandExecutedEvent = (BeforeCommandExecutedEventHandler)System.Delegate.Remove(BeforeCommandExecutedEvent, value);
            }
        }

        public delegate void AfterCommandExecutedEventHandler(object Sender, ReturnValueEventArgs e);
        private AfterCommandExecutedEventHandler AfterCommandExecutedEvent;

        public event AfterCommandExecutedEventHandler AfterCommandExecuted
        {
            add
            {
                AfterCommandExecutedEvent = (AfterCommandExecutedEventHandler)System.Delegate.Combine(AfterCommandExecutedEvent, value);
            }
            remove
            {
                AfterCommandExecutedEvent = (AfterCommandExecutedEventHandler)System.Delegate.Remove(AfterCommandExecutedEvent, value);
            }
        }

        public delegate void BeforeRemoveCommandEventHandler(string CommandName, System.ComponentModel.CancelEventArgs e);
        private BeforeRemoveCommandEventHandler BeforeRemoveCommandEvent;

        public event BeforeRemoveCommandEventHandler BeforeRemoveCommand
        {
            add
            {
                BeforeRemoveCommandEvent = (BeforeRemoveCommandEventHandler)System.Delegate.Combine(BeforeRemoveCommandEvent, value);
            }
            remove
            {
                BeforeRemoveCommandEvent = (BeforeRemoveCommandEventHandler)System.Delegate.Remove(BeforeRemoveCommandEvent, value);
            }
        }

        public delegate void BeforeRemoveAliasEventHandler(string AliasName, System.ComponentModel.CancelEventArgs e);
        private BeforeRemoveAliasEventHandler BeforeRemoveAliasEvent;

        public event BeforeRemoveAliasEventHandler BeforeRemoveAlias
        {
            add
            {
                BeforeRemoveAliasEvent = (BeforeRemoveAliasEventHandler)System.Delegate.Combine(BeforeRemoveAliasEvent, value);
            }
            remove
            {
                BeforeRemoveAliasEvent = (BeforeRemoveAliasEventHandler)System.Delegate.Remove(BeforeRemoveAliasEvent, value);
            }
        }

        public delegate void BeforeCaseSensitivityChangeEventHandler(object Sender, System.ComponentModel.CancelEventArgs e);
        private BeforeCaseSensitivityChangeEventHandler BeforeIgnoreCaseChangeEvent;

        public event BeforeCaseSensitivityChangeEventHandler BeforeCaseSensitivityChange
        {
            add
            {
                BeforeIgnoreCaseChangeEvent = (BeforeCaseSensitivityChangeEventHandler)System.Delegate.Combine(BeforeIgnoreCaseChangeEvent, value);
            }
            remove
            {
                BeforeIgnoreCaseChangeEvent = (BeforeCaseSensitivityChangeEventHandler)System.Delegate.Remove(BeforeIgnoreCaseChangeEvent, value);
            }
        }

        public delegate void AfterCaseSensitivityChangedEventHandler(object Sender, EventArgs e);
        private AfterCaseSensitivityChangedEventHandler AfterIgnoreCaseChangedEvent;

        public event AfterCaseSensitivityChangedEventHandler AfterCaseSensitivityChanged
        {
            add
            {
                AfterIgnoreCaseChangedEvent = (AfterCaseSensitivityChangedEventHandler)System.Delegate.Combine(AfterIgnoreCaseChangedEvent, value);
            }
            remove
            {
                AfterIgnoreCaseChangedEvent = (AfterCaseSensitivityChangedEventHandler)System.Delegate.Remove(AfterIgnoreCaseChangedEvent, value);
            }
        }
        #endregion


        // Attributes/Properties
        private CommandBuffer mobjCommandBuffer = new CommandBuffer();
        private Dictionary<string, CommandCallback> mobjCommands = new Dictionary<string, CommandCallback>();
        private Dictionary<string, string> mobjAlias = new Dictionary<string, string>();
        //private Console mobjConsole;

        /// <summary>
        /// Determines how casing will be handled.
        /// </summary>
        /// <remarks>The default value is true.</remarks>
        protected bool ignoreCase = true;


        /// <summary>
        /// Controls weather commads executed via the Execute method are check for case sensitivity.
        /// </summary>
        /// <value>Returns True if case sensitivity is turned on, False if it is not.</value>
        /// <remarks>If IgnoreCase is true and the command that is being run does not exactly match the casing of the actual 
        /// command name the command will still be executed. Only affects command and alias names. The default value is true.</remarks>
        public bool IgnoreCase
        {
            get { return ignoreCase; }
            set
            {
                if (value == ignoreCase) return;

                System.ComponentModel.CancelEventArgs e = new System.ComponentModel.CancelEventArgs(false);
                if (BeforeIgnoreCaseChangeEvent != null)
                    BeforeIgnoreCaseChangeEvent(this, e);
                if (e.Cancel) return;

                ignoreCase = value;

                if (AfterIgnoreCaseChangedEvent != null)
                    AfterIgnoreCaseChangedEvent(this, new EventArgs());
            }
        }

        /// <summary>
        /// Returns the name of a command for a givin command alias.
        /// </summary>
        /// <param name="AliasName">The name of the alias used to retrieve the command name.</param>
        /// <returns>The command associated with the alias. Returns null (Nothing in vb.net) if the alias anme does not exist.</returns>
        public string GetCommandFromAlias(string AliasName)
        {
            if (AliasExists(AliasName, this.ignoreCase) == false) return null;
            return mobjAlias[AliasName];
        }

        /// <summary>
        /// Returns true or false if an alias name is present.
        /// </summary>
        /// <param name="AliasName">The alias name to check.</param>
        /// <param name="ignoreCasing">If true casing will be ignored.</param>
        /// <returns>Returns true if the alias name is present.</returns>
        public bool AliasExists(string AliasName, bool ignoreCasing)
        {
            return DoAliasExists(AliasName, ignoreCasing) != null;
        }

        /// <summary>
        /// Returns true or false if an alias name is present.
        /// </summary>
        /// <param name="AliasName">The alias name to check.</param>
        /// <returns>Returns true if the alias name is present.</returns>
        /// <remarks>Ignores casing if the IgnoreCase property is true.</remarks>
        public bool AliasExists(string AliasName)
        {
            return DoAliasExists(AliasName, this.ignoreCase) != null;
        }

        /// <summary>
        /// Check if a command is present.
        /// </summary>
        /// <param name="CommandName">The command to check.</param>
        /// <returns>Returns true if the command is present.</returns>
        public bool CommandExists(string CommandName)
        {
            return CommandExists(CommandName, false, this.ignoreCase);
        }

        /// <summary>
        /// Check if a command is present.
        /// </summary>
        /// <param name="CommandName">The command to check.</param>
        /// <param name="searchAliases">If true CommandExists will also check the command alias list.</param>
        /// <returns>Returns true if the command is present.</returns>
        public bool CommandExists(string CommandName, bool searchAliases)
        {
            if ((DoCommandExists(CommandName, this.ignoreCase) == null) || searchAliases && (DoAliasExists(CommandName, this.IgnoreCase) == null)) return false;
            return true;
        }

        /// <summary>
        /// Check if a command is present.
        /// </summary>
        /// <param name="CommandName">The command to check.</param>
        /// <param name="searchAliases">If true CommandExists will also check the command alias list.</param>
        /// <param name="ignoreCase">If true casing will be ignored.</param>
        /// <returns>Returns true if the command is present.</returns>
        public bool CommandExists(string CommandName, bool searchAliases, bool ignoreCase)
        {
            if ((DoCommandExists(CommandName, ignoreCase) == null) || searchAliases && (DoAliasExists(CommandName, ignoreCase) == null)) return false;
            return true;
        }

        /// <summary>
        /// Returns true or false if an alias name is present.
        /// </summary>
        /// <param name="AliasName">The alias name to check.</param>
        /// <param name="ignoreCase">If true DoAliasExist will ignore casing.</param>
        /// <returns>Returns true if the alias name is present.</returns>
        protected internal string DoAliasExists(string AliasName, bool ignoreCase)
        {
            if (ignoreCase)
            {
                foreach (string C in mobjAlias.Keys)
                {
                    if (string.Compare(C, AliasName, ignoreCase) == 0) return C;
                }
                return null;
            }
            else
            {
                if (mobjAlias.ContainsKey(AliasName))
                {
                    return AliasName;
                }
                else
                {
                    return null;
                }
            }
        }

        /// <summary>
        /// Returns true or false if an command name is present.
        /// </summary>
        /// <param name="CommandName">The command name to check.</param>
        /// <param name="ignoreCase">If true DoCommandExists will ignore casing.</param>
        /// <returns>Returns true if the command name is present.</returns>
        protected internal CommandCallback DoCommandExists(string CommandName, bool ignoreCase)
        {
            if (ignoreCase)
            {
                foreach (string C in mobjCommands.Keys)
                {
                    if (string.Compare(C, CommandName, ignoreCase) == 0) return mobjCommands[C];
                }
            }
            else
            {
                if (mobjCommands.ContainsKey(CommandName)) return mobjCommands[CommandName];
            }
            return null;
        }

        /// <summary>
        /// Gets an array of category names for the different commands registered with the console.
        /// </summary>
        /// <returns>Returns an array of category names.</returns>
        public string[] GetCategoryList()
        {
            return GetCategoryList(false);
        }

        /// <summary>
        /// Gets an array of category names for the different commands registered with the console.
        /// </summary>
        /// <param name="includeHidden">If true will also include commands that have an <see cref="VisibleToUsers"/> attribute that is set to false.</param>
        /// <returns>Returns an array of category names.</returns>
        public string[] GetCategoryList(bool includeHidden)
        {
            // if no commands loaded return null
            if (mobjCommands.Count == 0) return null;

            StringCollection items = new StringCollection();

            // loop thru all commands
            foreach (string key in mobjCommands.Keys)
            {
                // get command
                CommandCallback cx;
                cx = mobjCommands[key];

                // check if category attribute was specified
                System.ComponentModel.CategoryAttribute cat;
                cat = Helpers.GetAttribute<System.ComponentModel.CategoryAttribute>(cx.Method);
                if (cat != null)
                {
                    // check if visibletousers attribute was specified
                    VisibleToUsers vis;
                    vis = Helpers.GetAttribute<VisibleToUsers>(cx.Method);
                    if (vis != null)
                    {
                        // check to add hidden
                        if (vis.Visible == false && includeHidden)
                        {
                            if (items.Contains(cat.Category) == false) items.Add(cat.Category);
                        }
                        else
                        {
                            if (items.Contains(cat.Category) == false) items.Add(cat.Category);
                        }
                    }
                    else
                    {
                        if (items.Contains(cat.Category) == false) items.Add(cat.Category);
                    }
                }
            }

            // copy commands to array and return
            string[] cmds;
            cmds = new string[items.Count];
            items.CopyTo(cmds, 0);
            items.Clear();
            items = null;
            return cmds;
        }

        /// <summary>
        /// Gets an array of commands names registered with the console.
        /// </summary>
        /// <returns>Returns an array of command names.</returns>
        public string[] GetCommandList()
        {
            return GetCommandList(string.Empty, this.ignoreCase, false);
        }

        /// <summary>
        /// Gets an array of commands names registered with the console.
        /// </summary>
        /// <param name="includeHidden">If true will also retrieve commands that have an <see cref="VisibleToUsers"/> attribute that is set to false.</param>
        /// <returns>Returns an array of command names.</returns>
        public string[] GetCommandList(bool includeHidden)
        {
            return GetCommandList(string.Empty, this.ignoreCase, includeHidden);
        }

        /// <summary>
        /// Gets an array of command names registered with the console.
        /// </summary>
        /// <param name="cat">The name of the category to retrieve commands from.</param>
        /// <returns>Returns an array of command names.</returns>
        public string[] GetCommandList(string cat)
        {
            return GetCommandList(cat, this.ignoreCase, false);
        }

        /// <summary>
        /// Gets an array of commands names registered with the console.
        /// </summary>
        /// <param name="includeHidden">If true will also retrieve commands that have an <see cref="VisibleToUsers"/> attribute this is set to false.</param>
        /// <param name="cat">The name of the category to retrieve commands from.</param>
        /// <returns>Returns an array of command names.</returns>
        public string[] GetCommandList(string cat, bool includeHidden)
        {
            return GetCommandList(cat, this.ignoreCase, includeHidden);
        }

        /// <summary>
        /// Gets an array of commands names registered with the console.
        /// </summary>
        /// <param name="includeHidden">If true will also retrieve commands that have an <see cref="VisibleToUsers"/> attribute this is set to false.</param>
        /// <param name="ignoreCase">If true checks for matching categories will be case insensitive.</param>
        /// <param name="category">The name of the category to retrieve commands from.</param>
        /// <returns>Returns an array of command names.</returns>
        /// <remarks>If category is string.Empty then all commands in all categories will be searched for.</remarks>
        public string[] GetCommandList(string category, bool ignoreCase, bool includeHidden)
        {
            // if no commands loaded return null
            if (mobjCommands.Count == 0) return null;

            // loop thru all commands
            StringCollection items = new StringCollection();
            foreach (string key in mobjCommands.Keys)
            {
                // get command
                CommandCallback cx;
                cx = mobjCommands[key];

                // try to get VisibleToUsers attribute
                VisibleToUsers vis;
                vis = Helpers.GetAttribute<VisibleToUsers>(cx.Method);

                // try to get Category attribute
                System.ComponentModel.CategoryAttribute cat;
                cat = Helpers.GetAttribute<System.ComponentModel.CategoryAttribute>(cx.Method);

                // check if command has been flagged invisible to console users and if we want to include the hidden commands
                if (vis != null && vis.Visible == false && includeHidden)
                {
                    // check if a category has been specified for command
                    if (cat != null && category != string.Empty)
                    {
                        // if category matches then add it
                        if (string.Compare(cat.Category, category, ignoreCase) == 0)
                        {
                            items.Add(key);
                        }
                    }
                    else // no category specified so add anyway
                    {
                        items.Add(key);
                    }
                }
                else // visibletousers attribute is not present or is set to true
                {
                    // check if a category has been specified for command
                    if (cat != null && category != string.Empty)
                    {
                        // if category matches then add it
                        if (string.Compare(cat.Category, category, ignoreCase) == 0)
                        {
                            items.Add(key);
                        }
                    }
                    else  // no category specified so add anyway
                    {
                        items.Add(key);
                    }
                }
            }

            // copy commands to array and return
            string[] cmds;
            cmds = new string[items.Count];
            items.CopyTo(cmds, 0);
            items.Clear();
            items = null;
            return cmds;
        }

        /// <summary>
        /// Gets an array of command aliases.
        /// </summary>
        /// <returns>Returns an array of command aliases.</returns>
        /// <remarks>Will return null (Nothing in vb.net) if there are no aliases.</remarks>
        public string[] GetAliasList()
        {
            if (mobjAlias.Count == 0) return null;

            string[] cmds;
            cmds = new string[mobjAlias.Count];
            mobjAlias.Keys.CopyTo(cmds, 0);
            return cmds;
        }

        /// <summary>
        /// Gets the number of command aliases.
        /// </summary>
        /// <returns>Returns the number of aliases in the aliases list.</returns>
        public int AliasCount
        {
            get { return mobjAlias.Count; }
        }

        /// <summary>
        /// Gets the number of commands registered with the console.
        /// </summary>
        /// <returns>Returns the number of commands registered with the console.</returns>
        public int CommandCount
        {
            get { return mobjCommands.Count; }
        }

        /// <summary>
        /// Returns the <see cref="CommandCallback"/> delegate for a specified command.
        /// </summary>
        /// <param name="CommandName">The name of the command to search for.</param>
        /// <param name="CheckAliases">If true the alias list will be checked as well.</param>
        /// <returns>Returns a reference to the <see cref="CommandCallback"/> delegate for the command. Returns null if the command could not be found.</returns>
        public CommandCallback GetCommand(string CommandName, bool CheckAliases)
        {
            CommandCallback Cmd;

            Cmd = DoCommandExists(CommandName, true);
            if (Cmd != null)
            {
                return Cmd;
            }
            if (CheckAliases)
            {
                string cName;
                cName = DoAliasExists(CommandName, true);
                if (cName == null)
                {
                    return null;
                }
                Cmd = DoCommandExists(mobjAlias[cName], true);
                if (Cmd != null)
                {
                    return Cmd;
                }

                //If _alias.ContainsKey(Cmd) Then
                //    Cmd = _alias(Cmd)
                //Else
                //    ' mobjManager.Console.Print("No command or command alias with that name exists!")
                //    'Exit Function
                //    Return Nothing
                //End If
            }
            else
            {
                return null;
            }

            return null; //_commands(Cmd)
        }

        /// <summary>
        /// Returns the command buffer that is responcible for buffering the commands send to the console.
        /// </summary>
        public CommandBuffer Buffer
        {
            get { return mobjCommandBuffer; }
        }

        /// <summary>
        /// CommandManager constructor.
        /// </summary>
        public CommandManager()
        {
        }

        ///// <summary>
        ///// CommandManager constructor.
        ///// </summary>
        ///// <param name="Console">A reference to a console object.</param>
        //public CommandManager(Console Console)
        //{
        //    mobjConsole = Console;
        //}

        /// <summary>
        /// CommandManager constructor.
        /// </summary>
        /// <param name="ignoreCasing">If true commands that are executed will not be checked for casing.</param>
        public CommandManager(bool ignoreCasing)
        {
            ignoreCase = ignoreCasing;
        }

        ///// <summary>
        ///// CommandManager constructor.
        ///// </summary>
        ///// <param name="Console">A reference to a console object.</param>
        ///// <param name="ignoreCasing">If true commands that are executed will not be checked for casing.</param>
        //public CommandManager(Console Console, bool ignoreCasing)
        //{
        //    mobjConsole = Console;
        //    ignoreCase = ignoreCasing;
        //}

        // Operations/Methods
        //Public Sub Initialize()
        //     _HasBeenInit = True
        //End Sub

        /// <summary>
        /// Adds a command alias to the alias list.
        /// </summary>
        /// <param name="AliasName">The name for the alias.</param>
        /// <param name="Command">The command to be executed.</param>
        public void AddAlias(string AliasName, string Command)
        {

            // check if valid alias name
            AliasName = AliasName.Trim();
            if (Helpers.IsValidName(AliasName) == false)
            {
                throw (new ArgumentException(Resources.AliasNameIsNotValid.Replace(Resources.RKEY, AliasName), "AliasName"));
            }

            // check if alias already exists
            if (AliasExists(AliasName, true))
            {
                throw (new ArgumentException(Resources.AliasNameAlreadyExists.Replace(Resources.RKEY, AliasName), "AliasName"));
            }
            else
            {
                // add alias
                mobjAlias.Add(AliasName, Command);
            }
        }

        /// <summary>
        /// Adds a command to the command list.
        /// </summary>
        /// <param name="CommandName">The name of the comamnd.</param>
        /// <param name="callback">A reference to a <see cref="CommandCallback"/> delegate that will be executed when the command is called.</param>
        /// <exception cref="ArgumentException">Is thrown if the command name is invalid or already exists.</exception>
        /// <exception cref="ArgumentNullException">Is thrown if the callback is null (Nothing in vb.net).</exception>
        public void AddCommand(string CommandName, CommandCallback callback) //, ByVal processor As CommandProcessor)
        {
            CommandName = CommandName.Trim();
            if (Helpers.IsValidName(CommandName) == false)
            {
                throw (new ArgumentException(Resources.InvalidCharacters, "CommandName"));
            }

            if (callback == null)
            {
                throw (new ArgumentNullException("Callback"));
            }
            else
            {
                if (CommandExists(CommandName))
                {
                    throw (new ArgumentException(Resources.CommandNameAlreadyExists.Replace(Resources.RKEY, CommandName), "CommandName"));
                }
                else
                {
                    mobjCommands.Add(CommandName, callback);
                }
            }
        }

        private bool DoRemoveCommand(string CommandName)
        {
            System.ComponentModel.CancelEventArgs e = new System.ComponentModel.CancelEventArgs(false);
            try
            {
                if (BeforeRemoveCommandEvent != null)
                    BeforeRemoveCommandEvent(CommandName, e);
            }
            catch
            {
            }
            if (e.Cancel)
            {
                return false;
            }
            mobjCommands[CommandName] = null;
            mobjCommands.Remove(CommandName);
            return true;
        }

        /// <summary>
        /// Removes a command from the list.
        /// </summary>
        /// <param name="CommandName">The name of the command to be removed.</param>
        /// <exception cref="ArgumentException">Is thrown if the command name is invalid or does not exists.</exception>
        public void RemoveCommand(string CommandName)
        {
            if (this.ignoreCase)
            {
                foreach (string C in mobjCommands.Keys)
                {
                    if (string.Compare(C, CommandName, this.ignoreCase) == 0)
                    {
                        DoRemoveCommand(C);
                        return;
                    }
                }
                throw (new ArgumentException(Resources.SpecifiedCommandNotExist, "CommandName"));
            }
            else
            {
                DoRemoveCommand(CommandName);
            }
        }

        /// <summary>
        /// Removes a command alias.
        /// </summary>
        /// <param name="AliasName">The name if the name of the command alias.</param>
        /// <returns>Will return False if the <see cref="BeforeRemoveAlias"/> event was canceled, otherwise will return True.</returns>
        private bool DoRemoveAlias(string AliasName)
        {
            System.ComponentModel.CancelEventArgs e = new System.ComponentModel.CancelEventArgs(false);

            if (BeforeRemoveAliasEvent != null)
                BeforeRemoveAliasEvent(AliasName, e);

            if (e.Cancel) return false;

            mobjAlias.Remove(AliasName);
            return true;
        }

        /// <summary>
        /// Removes a command alias from the list.
        /// </summary>
        /// <param name="AliasName">The name of the alias to be removed.</param>
        /// <exception cref="ArgumentException">Is thrown if the alias name is invalid or does not exists.</exception>
        public void RemoveAlias(string AliasName)
        {
            this.RemoveAlias(AliasName, this.ignoreCase);
        }

        /// <summary>
        /// Removes a command alias from the list.
        /// </summary>
        /// <param name="AliasName">The name of the alias to be removed.</param>
        /// <param name="ignoreCase">If true RemoveAlias will ignore casing.</param>
        /// <exception cref="ArgumentException">Is thrown if the alias name is invalid or does not exists.</exception>
        public void RemoveAlias(string AliasName, bool ignoreCase)
        {
            if (ignoreCase)
            {
                foreach (string C in mobjAlias.Keys)
                {
                    if (string.Compare(C, AliasName.Trim(), this.ignoreCase) == 0)
                    {
                        DoRemoveAlias(C);
                        return;
                    }
                }
                throw (new ArgumentException(Resources.AliasNameIsNotValid.Replace(Resources.RKEY, AliasName), "AliasName"));
            }
            else
            {
                DoRemoveAlias(AliasName.Trim());
            }
        }

        #region Execute method overloads ...
        /// <summary>
        /// Executes a collection of commands.
        /// </summary>
        /// <param name="Commands">The collection of commands to be executed.</param>
        public void Execute(StringCollection Commands)
        {
            this.Execute(Commands, null);
        }

        /// <summary>
        /// Executes a collection of commands.
        /// </summary>
        /// <param name="Commands">The collection of commands to be executed.</param>
        /// <param name="ReturnValueCB">A reference to a <see cref="ReturnValueCallback"/> delegate that will be called that will report the command, 
        /// the return value, and the index of the command that was executed.</param>
        public void Execute(StringCollection Commands, ReturnValueCallback ReturnValueCB)
        {
            if (Commands == null) return;

            for (int idx = 0; idx < Commands.Count; idx++)
            {
                string cmd = Commands[idx];
                if (ReturnValueCB != null)
                {
                    string val = null;
                    val = this.DoExecuteString(cmd, true, true);
                    ReturnValueCB(val, cmd, idx);
                }
                else
                {
                    this.DoExecuteString(cmd, true, false);
                }
            }
        }

        /// <summary>
        /// Executes an array of commands.
        /// </summary>
        /// <param name="Commands">The array of commands to be executed.</param>
        public void Execute(string[] Commands)
        {
            this.Execute(Commands, null);
        }

        /// <summary>
        /// Executes an array of commands.
        /// </summary>
        /// <param name="Commands">The array of commands to be execute.</param>
        /// <param name="ReturnValueCB">A reference to a <see cref="ReturnValueCallback"/> delegate that will be called that will report the command, 
        /// the return value, and the index of the command that was executed.</param>
        public void Execute(string[] Commands, ReturnValueCallback ReturnValueCB)
        {
            // check if it is a empty array
            if (Commands == null) return;

            for (int idx = 0; idx < Commands.Length; idx++)
            {
                string cmd = Commands[idx];
                if (ReturnValueCB != null)
                {
                    string val = null;
                    val = this.DoExecuteString(cmd, true, true);
                    ReturnValueCB(val, cmd, idx);
                }
                else
                {
                    this.DoExecuteString(cmd, true, false);
                }
            }
        }

        /// <summary>
        /// Executes a specified command.
        /// </summary>
        /// <param name="Command">The command to be executed</param>
        public void Execute(string Command)
        {
            this.DoExecuteString(Command, true, false);
        }

        /// <summary>
        /// Executes a specified command.
        /// </summary>
        /// <param name="Command">The command to be executed.</param>
        /// <param name="usedAsFunction">If true, informs the command that is to be executed that it is being treated like a function.</param>
        /// <returns>Returns the return value of the command.</returns>
        public string Execute(string Command, bool usedAsFunction)
        {
            return this.DoExecuteString(Command, true, usedAsFunction);
        }

        /// <summary>
        /// Executes all commands stored in the command buffer.
        /// </summary>
        public void Execute()
        {
            this.Execute(0);
        }

        /// <summary>
        /// Executes all commands stored in the command buffer.
        /// </summary>
        /// <param name="ReturnValueCB">A reference to a <see cref="ReturnValueCallback"/> delegate that will be called that will report the command, 
        /// the return value, and the index of the command that was executed.</param>
        public void Execute(ReturnValueCallback ReturnValueCB)
        {
            this.Execute(0, ReturnValueCB);
        }

        /// <summary>
        /// Executes a specified number of commands stored in the command buffer.
        /// </summary>
        /// <param name="Count">The number of commands to be executed. If a value less then 1 is specified all commands stored in the 
        /// buffer will be executed.</param>
        public void Execute(int Count)
        {
            this.Execute(Count, null);
        }

        /// <summary>
        /// Executes a specified number of commands stored in the command buffer.
        /// </summary>
        /// <param name="Count">The number of commands to be executed. If a value less then 1 is specified all commands stored
        /// in the buffer will be executed.</param>
        /// <param name="ReturnValueCB">A reference to a <see cref="ReturnValueCallback"/> delegate that will be called, that will report the command, 
        /// the return value, and the index of the command that was executed.</param>
        public void Execute(int Count, ReturnValueCallback ReturnValueCB)
        {
            int idx;
            if (Count < 1) Count = mobjCommandBuffer.Lines.Count;
            if (Count > mobjCommandBuffer.Lines.Count) Count = mobjCommandBuffer.Lines.Count;

            // process commands
            for (idx = 0; idx < Count; idx++)
            {
                try
                {
                    string cmd = mobjCommandBuffer.Lines.Dequeue();
                    if (ReturnValueCB != null)
                    {
                        string val = null;
                        val = this.DoExecuteString(cmd, true, true);
                        ReturnValueCB(val, cmd, idx);
                    }
                    else
                    {
                        this.DoExecuteString(cmd, true, false);
                    }
                }
                catch { }
            }
        }
        #endregion  // execute method overloads

        private bool RaiseBeforeCommandExecuted(string Command, string[] Args, string ReturnValue, bool Cancel)
        {
            ReturnValueEventArgs e = new ReturnValueEventArgs(Command, Args, ReturnValue, Cancel);
            if (BeforeCommandExecutedEvent != null)
                BeforeCommandExecutedEvent(this, e);
            return e.Cancel;
        }

        private bool RaiseAfterCommandExecuted(string commandString, string Command, string[] Args, string ReturnValue, bool Cancel)
        {
            ReturnValueEventArgs e = new ReturnValueEventArgs(commandString, Command, Args, ReturnValue, Cancel);
            if (AfterCommandExecutedEvent != null)
                AfterCommandExecutedEvent(this, e);
            return e.Cancel;
        }

        private string DoExecuteString(string CommandString, bool RaiseExecutedEvent, bool usedAsFunction)
        {
            Command command;
            string Ret = null;

            // retreve the command from the text
            command = ParseCommandText(CommandString);

            // check to make sure call back exists
            if (command != null && command.Callback != null)
            {
                // check to raise event
                if (RaiseExecutedEvent)
                {
                    // raise event and return nothig if canceled
                    if (RaiseBeforeCommandExecuted(command.Name, command.Arguments, null, false))
                    {
                        return null;
                    }
                }

                // call the delegate for the command               
                command.Arguments = this.ProcessArguments(command.Arguments);

                // check if the object implements ICommandExtender 
                ICommandExtender tmp = null;
                try
                {
                    tmp = (ICommandExtender)command.Callback.Target;//.GetType().GetInterface(typeof(ICommandExtender).Name);
                }
                catch { }

                // if tmp is not null it implements ICommandExtender 
                if (tmp != null)
                {
                    // only call the method if it is connected.
                    if (tmp.IsConnected)
                    {
                        Ret = command.Callback(command.Arguments, usedAsFunction);
                    }
                }
                else
                {
                    // call the method if it does not implement the ICommandExtender interface.
                    Ret = command.Callback(command.Arguments, usedAsFunction);
                }

                // check to raise event
                if (RaiseExecutedEvent)
                {
                    // raise event and return nothing if canceled
                    if (RaiseAfterCommandExecuted(CommandString, command.Name, command.Arguments, Ret, false))
                    {
                        return null;
                    }
                }
            }

            return Ret;
        }

        /// <summary>
        /// Processes a collection of arguments.
        /// </summary>
        /// <param name="Args">The collection of arguments to be processed.</param>
        /// <returns>Returns an array containing the processed arguments.</returns>
        /// <remarks>If the collection of arguments contains function calls, the function calls will be processed and replaced with there literal values.</remarks>
        public string[] ProcessArguments(StringCollection Args)
        {
            StringCollection Params = new StringCollection();

            if (Args == null) return null;

            // process args
            while (Args.Count > 0)
            {
                if (Args[0].Trim() == this.startFunction)
                {
                    Args.RemoveAt(0);
                    try
                    {
                        string[] Value;
                        Value = ProcessArguments(Args);
                       // if (Value != null && Value.Length > 0 && Value[0] != null) Value[0] = Value[0].TrimStart(' ');
                        Params.AddRange(Value);

                    }
                    catch (ArgumentNullException)
                    {
                        // the returned array was nothing
                    }

                }
                else if (Args[0].Trim() == this.endFunction)
                {

                    Args.RemoveAt(0);
                     string arg =null;
                   if (Args.Count > 0)  arg = Args[0];
                    if (arg != null &&  arg.StartsWith(" ")) Args[0] = arg.Substring(1);
                    string pValue;
                    pValue = this.DoExecuteString(Helpers.StringCollectionToString(Params, string.Empty), true, true);

                    return new string[] { pValue };
                }
                else
                {

                    Params.Add(Args[0]);
                    Args.RemoveAt(0);
                }
            }

            // return new args
            return Helpers.StringCollectionToArray(Params);
        }

        /// <summary>
        /// Will process the argument array for any function definitions, converting them to literal values.
        /// </summary>
        /// <param name="Args">A list of arguments to process.</param>
        /// <returns>Will return the converted function call arguments into there literal values.</returns>
        /// <remarks>
        /// If the arguments of an echo command was "echo [ GetSomeValue ]" then the GetSomeValue command would be called like a 
        /// function and would be replaced with it's return value. So if it's return value was "123" the arguments for the echo command
        /// would look like "echo 123" after processing. 
        /// </remarks>
        public string[] ProcessArguments(string[] Args)
        {
            if (Args == null) return null;

            StringCollection SC = new StringCollection();
            SC.AddRange(Args);
            string[] NewArgs;
            NewArgs = ProcessArguments(SC);
            SC = null;
            return NewArgs;
        }

        private Command ParseCommandText(string commandText)
        {
            if (commandText == null) return null;
            Command command = new Command();

            commandText = commandText.TrimStart(' ');
            string name = "";
            StringCollection arguments = new StringCollection();
            if (commandText == "")
            {
                command.Callback = null;
                return null;
            }

            int index = commandText.IndexOf(" ");
            if (index != -1)
            {
                // we have arguments to parse out
                name = commandText.Substring(0, index);
                string parameters = commandText.Remove(0, index + 1);
                //   arguments.AddRange(parameters.Split(' '));

                // build arguments
                while (parameters != string.Empty)
                {
                    index = parameters.Length;
                    for (int idx = 0; idx < parameters.Length; idx++)
                    {
                        bool found = false;
                        if (parameters[idx] != ' ')// && ((idx + 1 < parameters.Length && parameters[idx + 1] == ' ') || idx + 1 == parameters.Length))
                        {
                            for (int i = idx; i < parameters.Length; i++)
                            {
                                if (parameters[i] == ' ')
                                {
                                    index = i;
                                    found = true;
                                    break;
                                }
                            }
                            if (found) break;
                        }
                    }

                    // check if there are any remaining parameters
                    if (parameters.Trim() == string.Empty)
                    {
                        // check if there were any previous parameters added
                        if (arguments.Count == 0)
                        {
                            // user only typed in spaces?
                            arguments.Add(parameters);
                        }
                        else
                        {
                            // append trailing spaces to last parameter
                            arguments[arguments.Count - 1] += parameters;
                        }

                        parameters = string.Empty;
                    }
                    else
                    {
                        // add argument
                        arguments.Add(parameters.Substring(0, index));
                        // crop parameters
                        parameters = parameters.Substring(index);
                    }
                }
            }
            else
            {
                // there are no arguments so all we have is the command name
                name = commandText.TrimStart();
                //arguments = null;
            }

            command.Name = name;
            command.Callback = DoCommandExists(name, true);
            if (command.Callback == null)
            {
                string cName;
                cName = DoAliasExists(name, true);
                if (cName != null)
                {
                    command = this.ParseCommandText(mobjAlias[cName]);
                    if (command.Arguments != null)
                    {
                        for (int idx = 0; idx < command.Arguments.Length; idx++)
                        {
                            arguments.Insert(idx, command.Arguments[idx]);
                        }
                    }
                }
            }

            // set arguments
            string[] tmp = null;
            if (arguments != null && arguments.Count > 0)
            {
                Array.Resize<string>(ref tmp, arguments.Count);
                arguments.CopyTo(tmp, 0);
            }
            command.Arguments = tmp;
            return command;
        }

        #region " IDisposable Support "
        private bool disposedValue = false; // To detect redundant calls

        // IDisposable
        protected virtual void Dispose(bool disposing)
        {
            if (!this.disposedValue)
            {
                if (disposing)
                {
                    // free unmanaged resources when explicitly called
                    mobjCommandBuffer.Dispose();
                    mobjCommandBuffer = null;
                    mobjCommands.Clear();
                    mobjCommands = null;
                    mobjAlias.Clear();
                    mobjAlias = null;
                    //mobjConsole = null;
                }

                // free shared unmanaged resources
            }
            this.disposedValue = true;
        }

        // This code added by Visual Basic to correctly implement the disposable pattern.
        public void Dispose()
        {
            // Do not change this code.  Put cleanup code in Dispose(ByVal disposing As Boolean) above.
            Dispose(true);
            GC.SuppressFinalize(this);
        }
        #endregion

    }
}
