/*
Created by: X
http://www.createdbyx.com/

xGameConsoleXNA is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
using System.Diagnostics;
//using System.Data;
using System.Collections;
using System.Collections.Generic;
using System;


namespace xGameConsoleXNA
{
 
    /// <summary>
    /// Manages a list of objects that implement the <see cref="ICommandExtender"/> interface.
    /// </summary>
    public class CommandExtenderManager : IDisposable
    {

        private List<ICommandExtender> mobjExtentions;
        private CommandManager mobjCommandManager;
        private bool mblnDisconnectOnDispose = true;


        /// <summary>
        /// Removes all objects from the list.
        /// </summary>
        public void Clear()
        {
            this.mobjExtentions.Clear();
        }

        /// <summary>
        /// Searches a privided assembly file for objects that implement the <see cref="ICommandExtender"/> interface, and attempts to create instances of them. 
        /// </summary>
        /// <param name="AssemblyFile">The filename of the assembly to read.</param>
        /// <param name="ExceptionCallback">This delegate will be called if there is a error when trying to create an instance of the object.</param>
        /// <param name="ExtenderLoadedCallback">This delegate will be called if the object was created successfully. A reference to the object will also be passed to this delegate.</param>
        /// <returns>Returns a list of successfully created objects that implement the <see cref="ICommandExtender"/> interface.</returns>
        public static List<ICommandExtender> LoadExtenderObjects(string AssemblyFile, LoadExtenderException ExceptionCallback, ICommandExtenderLoaded ExtenderLoadedCallback)
        {
            List<ICommandExtender> Items = new List<ICommandExtender>();
            try
            {
                System.Reflection.Assembly Asm;
                Asm = System.Reflection.Assembly.LoadFrom(AssemblyFile);
                Type[] Types;
                Types = Asm.GetTypes();
                foreach (Type T in Types)
                {
                    // make sure type is a public class
                    if (T.IsClass && T.IsPublic)
                    {
                        //check if type implements extender interface
                        if (T.GetInterface(typeof(ICommandExtender).FullName) == null == false)
                        {
                            // make sure type  has constructor that does not have any parameters (simple Sub New() constructor)
                            System.Reflection.ConstructorInfo Con;
                            Con = T.GetConstructor(Type.EmptyTypes);
                            if (Con != null)
                            {
                                // we can now attempt to create the object
                                ICommandExtender Obj = null;
                                try
                                {
                                    Obj = (ICommandExtender)Activator.CreateInstance(T, null);
                                }
                                catch (Exception e)
                                {
                                    ExceptionCallback.Invoke(e);
                                }

                                if (Obj != null)
                                {
                                    // successfully created object so add item to list
                                    Items.Add(Obj);
                                }

                                // report back that the extender was loaded.
                                ExtenderLoadedCallback.Invoke(Obj);
                            }
                        }
                    }
                }

            }
            catch (Exception ex)
            {
                // this exception may be thrown because the assembly may fail to load due to a security exception etc
                ExceptionCallback.Invoke(ex);
            }

            //return the list of items loaded
            return Items;
        }

        /// <summary>
        /// Adds an array of objects that implement the <see cref="ICommandExtender"/> interface.
        /// </summary>
        /// <param name="Ext">The array of objects to be added.</param>
        /// <param name="CallConnect">If true the OnConnect method will be called to connect the object.</param>
        public void Add(ICommandExtender[] Ext, bool CallConnect)
        {
            foreach (ICommandExtender e in Ext)
            {
                if (e != null) this.Add(e, CallConnect);
            }
        }

        /// <summary>
        /// Calls the OnDisconnect method on each of the stored objects.
        /// </summary>
        public void DisconnectAll()
        {
            foreach (ICommandExtender e in mobjExtentions)
            {
                if (e != null) e.OnDisconnect();
            }
        }

        /// <summary>
        /// Calls the OnConnect method on each of the stored objects.
        /// </summary>
        public void ConnectAll()
        {
            foreach (ICommandExtender e in mobjExtentions)
            {
                if (e != null) e.OnConnect(this.mobjCommandManager, this);
            }
        }

        /// <summary>
        /// Adds a object implementing the <see cref="ICommandExtender"/> interface.
        /// </summary>
        /// <param name="Ext">Any object implementing the <see cref="ICommandExtender"/> interface.</param>
        public int Add(ICommandExtender Ext)
        {
            return this.Add(Ext, true);
        }

        /// <summary>
        /// Adds a array of objects implementing the <see cref="ICommandExtender"/> interface.
        /// </summary>
        /// <param name="Ext">An array of objects implementing the <see cref="ICommandExtender"/> interface.</param>
        public void Add(ICommandExtender[] Ext)
        {
            this.Add(Ext, true);
        }

        /// <summary>
        /// Adds a object implementing the <see cref="ICommandExtender"/> interface.
        /// </summary>
        /// <param name="Ext">Any object implementing the <see cref="ICommandExtender"/> interface.</param>
        /// <param name="CallConnect">If true will call the OnConnect method of the object but only if the IsConnected property is false.</param>
        /// <returns>Returns -1 if the Ext Param was Null (Nothing in vb.net).<br/>
        /// Returns -2 if the OnConnect method threw an exception.<br/>
        /// Will return a value of 0 or heigher indicating the indexed location of the added object.</returns>
        /// <remarks></remarks>
        public int Add(ICommandExtender Ext, bool CallConnect)
        {
            if (Ext == null)
            {
                return -1;
            }
            int Index;
            mobjExtentions.Add(Ext);
            Index = mobjExtentions.IndexOf(Ext);
            if (Ext.IsConnected == false && CallConnect)
            {
                try
                {
                    Ext.OnConnect(mobjCommandManager, this);
                }
                catch
                {
                    mobjExtentions.RemoveAt(Index);
                    return -2;
                }
            }
            return Index;
        }

        /// <summary>
        /// Returns the number of objects that have been stored.
        /// </summary>
        public int Count
        {
            get { return mobjExtentions.Count; }
        }

        /// <summary>
        /// Returns a object from the list.
        /// </summary>
        /// <param name="Index">The index of the object to be returned.</param>
        /// <returns>Returns a reference to a object that implements the <see cref="ICommandExtender"/> interface.</returns>
        /// <exception cref="ArgumentOutOfRangeException">if the specified index was out of range.</exception>
        public ICommandExtender GetExtender(int Index)
        {
            if (Index < 0 || Index > mobjExtentions.Count - 1)
            {
                throw (new ArgumentOutOfRangeException("Index"));
            }

            return ((ICommandExtender)mobjExtentions[Index]);
        }

        /// <summary>
        /// Removes a object from the list.
        /// </summary>
        /// <param name="Index">The index of the object to be removed.</param>
        /// <param name="CallDisconnect">If true, the OnDisconnect method will be called before the object is removed from the list.</param>
        /// <exception cref="ArgumentOutOfRangeException">If the specified index was out of range.</exception>
        public void Remove(int Index, bool CallDisconnect)
        {
            if (Index < 0 || Index > mobjExtentions.Count - 1)
            {
                throw (new ArgumentOutOfRangeException("Index"));
            }
            if (CallDisconnect && mobjExtentions[Index].IsConnected)
            {
                mobjExtentions[Index].OnDisconnect();
            }
            mobjExtentions.RemoveAt(Index);
        }

        /// <summary>
        /// Removes a object from the list.
        /// </summary>
        /// <param name="Extention">A reference to an object that is already stored in the list.</param>
        /// <param name="CallDisconnect">If true, the OnDisconnect method will be called before the object is removed from the list.</param>
        /// <exception cref="ArgumentOutOfRangeException">If the specified index was out of range.</exception>
        public void Remove(ICommandExtender Extention, bool CallDisconnect)
        {
            int Index;
            Index = this.mobjExtentions.IndexOf(Extention);
            if (Index < 0 || Index > mobjExtentions.Count - 1)
            {
                throw (new ArgumentOutOfRangeException("Index"));
            }

            if (CallDisconnect && mobjExtentions[Index].IsConnected)
            {
                mobjExtentions[Index].OnDisconnect();
            }
            mobjExtentions.RemoveAt(Index);
        }

        /// <summary>
        /// CommandExtenderManager constructor.
        /// </summary>
        /// <param name="Manager">A reference to a CommandManager object.</param>
        /// <exception cref="ArgumentNullException">Cannot set CommandManager to null. CommandManager is needed when connecting extender objects.</exception>
        public CommandExtenderManager(CommandManager Manager)
        {
            if (Manager == null) throw new ArgumentNullException("Manager");
            mobjCommandManager = Manager;
            mobjExtentions = new List<ICommandExtender>();
        }

        #region " IDisposable Support "
        private bool disposed = false;
        // IDisposable
        private void Dispose(bool disposing)
        {
            if (!this.disposed)
            {
                if (disposing)
                {
                    if (mblnDisconnectOnDispose)  this.DisconnectAll();
                    
                    mobjExtentions.Clear();
                    mobjExtentions = null;
                    mobjCommandManager = null;
                }
                // put code to free unmanaged resources here
            }
            this.disposed = true;
        }

        // This code added by Visual Basic to correctly implement the disposable pattern.
        public virtual void Dispose()
        {
            // Do not change this code.  Put cleanup code in Dispose(ByVal disposing As Boolean) above.
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        ~CommandExtenderManager()
        {
            // Do not change this code.  Put cleanup code in Dispose(ByVal disposing As Boolean) above.
            Dispose(false);
            //base.Finalize();
        }
        #endregion

        /// <summary>
        /// Gets/Sets a reference to the ComandManager.
        /// </summary>
        /// <exception cref="NullReferenceException">Cannot set CommandManager to null. CommandManager is needed when connecting extender objects.</exception>
        public CommandManager CommandManager
        {
            get { return mobjCommandManager; }
            set {
                if (value == null) throw new NullReferenceException();
                mobjCommandManager = value; }
        }

        /// <summary>
        /// If set to true and this objects Dispose method is called, all stored objects will have there OnDisconnect methods called.
        /// </summary>
        public bool CallDisconnectOnDispose
        {
            get { return mblnDisconnectOnDispose; }
            set { mblnDisconnectOnDispose = value; }
        }
    }

}
