/*
Created by: X
http://www.createdbyx.com/

xGameConsoleXNA is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/

Thanks goes out to Shawn Hargreaves who provided info on how to embed game assets in your compiled assemblies.
Shawn Hargreaves
http://blogs.msdn.com/shawnhar/default.aspx
"Embedding content as resources"
http://blogs.msdn.com/shawnhar/archive/2007/06/12/embedding-content-as-resources.aspx
*/

#region Using Statements
using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Content;
using System.Reflection;
#endregion

namespace xGameConsoleXNA
{
    /// <summary>
    /// This is a game component that inherits DrawableGameComponent.
    /// </summary>
    public partial class ConsoleComponent : Microsoft.Xna.Framework.DrawableGameComponent
    {
        /// <summary>
        /// Defines different opening and closing states for the console.
        /// </summary>
        public enum ConsoleState
        {
            /// <summary>
            /// The console is closed.
            /// </summary>
            Closed,
            /// <summary>
            /// The console is closing.
            /// </summary>
            Closing,
            /// <summary>
            /// The console is opening.
            /// </summary>
            Opening,
            /// <summary>
            /// The console is open.
            /// </summary>
            Open
        }

        #region Fields ...
        /// <summary>
        /// The background color for the default console renderer.
        /// </summary>
        protected Color backColor = new Color(0, 0, 0, 128);
        /// <summary>
        /// The font used when drawing the text for the default renderer.
        /// </summary>
        protected internal SpriteFont font;
        /// <summary>
        /// The border color for the default console renderer.
        /// </summary>
        protected Color borderColor = Color.White;
        /// <summary>
        /// The text color for the default console renderer.
        /// </summary>
        protected Color textColor = Color.White;
        /// <summary>
        /// If true the built in console will be animated when it opens and closes.
        /// </summary>
        protected internal bool mblnAnimateConosle = true;
        private VertexBuffer vb;
        private BasicEffect fx;
        private Rectangle bounds = new Rectangle(0, 0, 600, 400);
        private Console console;
        private RenderConsole rendermethod;
        private CommandManager commandManager;
        private VertexDeclaration decl;
        private Keys displayToggle = Keys.Escape;
        long RenderConsole_LastCursonBlinkTime;
        bool RenderConsole_CursorVisible;
        private CommandExtenderManager extenderManager;
        internal ResourceContentManager content;
        internal StringCollection commandHistory = new StringCollection();
        internal int historyIndex = -1;
        private int maxCommandHistory = 100;
        private int insertIndex;
        private SpriteBatch sb;
        private bool wordWrap = true;
        private Vector2 maxScrollValues;
        private Vector2 scrollValues = Vector2.Zero;
        private float msngCurrentConsolePercentage = 0.0F;
        private float msngMaxConsolePercentage = 100.0F;
        private float msngConsoleSpeed = 1f;
        private ConsoleState mintConsoleState = ConsoleState.Closed;
        private IUserInputService input;
        // caps lock state is set to false because it is the most likley state that it will be in when the app is run
        bool capsPressed = false;
        private PlayerIndex gamepadControl = PlayerIndex.One;
        private bool allowGamepadControl = true;

        #endregion


        /// <summary>
        /// Gets/Sets weather or not a user can use a XBox360 game pad to control the console.
        /// </summary>
        public bool AllowGamepadControl
        {
            get { return allowGamepadControl; }
            set { allowGamepadControl = value; }
        }

        /// <summary>
        /// Gets/Sets the player that has control of the console when using a XBox360 gamepad.
        /// </summary>
        public PlayerIndex PlayerControl
        {
            get { return gamepadControl; }
            set { gamepadControl = value; }
        }

        /// <summary>
        /// Gets/Sets weather or not the console will be animated when opening and closing.
        /// </summary>
        public bool AnimateConosle
        {
            get { return mblnAnimateConosle; }
            set { mblnAnimateConosle = value; }
        }

        private void Render(GameTime gameTime)
        {
            if (!this.IsVisible) return;


            if (this.mblnAnimateConosle)
            {
                /// update console state
                if (mintConsoleState == ConsoleState.Opening && msngCurrentConsolePercentage <= msngMaxConsolePercentage)
                {
                    if (msngConsoleSpeed <= 0)
                    {
                        msngCurrentConsolePercentage = msngMaxConsolePercentage;
                        mintConsoleState = ConsoleState.Open;
                    }
                    else
                    {

                        msngCurrentConsolePercentage += msngConsoleSpeed * gameTime.ElapsedGameTime.Milliseconds;

                        if (msngCurrentConsolePercentage >= msngMaxConsolePercentage)
                        {

                            msngCurrentConsolePercentage = msngMaxConsolePercentage;
                            mintConsoleState = ConsoleState.Open;

                        }
                    }

                }
                else if (mintConsoleState == ConsoleState.Closing && msngCurrentConsolePercentage >= 0.0F)
                {
                    if (msngConsoleSpeed <= 0)
                    {
                        msngCurrentConsolePercentage = 0.0F;
                        mintConsoleState = ConsoleState.Closed;
                    }
                    else
                    {

                        msngCurrentConsolePercentage -= msngConsoleSpeed * gameTime.ElapsedGameTime.Milliseconds;

                        if (msngCurrentConsolePercentage <= 0.0F)
                        {

                            msngCurrentConsolePercentage = 0.0F;
                            mintConsoleState = ConsoleState.Closed;
                        }
                    }
                }
                else
                {
                    if (mintConsoleState == ConsoleState.Opening)
                    {
                        msngCurrentConsolePercentage = msngMaxConsolePercentage;
                        mintConsoleState = ConsoleState.Open;
                    }
                    else if (mintConsoleState == ConsoleState.Closing)
                    {
                        msngCurrentConsolePercentage = 0.0F;
                        mintConsoleState = ConsoleState.Closed;
                    }
                }

            }
            else // non animated console
            {


                if (mintConsoleState == ConsoleState.Opening)
                {
                    msngCurrentConsolePercentage = msngMaxConsolePercentage;
                    mintConsoleState = ConsoleState.Open;
                }
                else if (mintConsoleState == ConsoleState.Closing)
                {
                    msngCurrentConsolePercentage = 0.0F;
                    mintConsoleState = ConsoleState.Closed;
                }
            }


            // call the render console delegate is present.
            if (this.rendermethod != null)
            {
                this.DoRender(gameTime);
            }

        }

        /// <summary>
        /// Gets/Sets the visibility state of the console.
        /// </summary>
        public ConsoleState State
        {
            get { return mintConsoleState; }
            set { mintConsoleState = value; }
        }

        /// <summary>
        /// Gets/Sets the speed at which the console will open or close.
        /// </summary>
        /// <remarks>Has no effect if the AnimateConsole property is set to false.</remarks>
        public float ConsoleSpeed
        {
            get { return msngConsoleSpeed; }
            set { msngConsoleSpeed = value; }
        }

        protected override void Dispose(bool disposing)
        {
            base.Dispose(disposing);
            this.rendermethod = null;
            this.input = null;
        }

        /// <summary>
        /// Toggles the state of the console from open to closed.
        /// </summary>
        public void ToggleState()
        {
            if (mintConsoleState == ConsoleState.Closed || mintConsoleState == ConsoleState.Closing)
            {
                mintConsoleState = ConsoleState.Opening;
            }
            else
            {
                this.console.CancelQuestion();
                mintConsoleState = ConsoleState.Closing;
            }
        }

        /// <summary>
        /// Shows the console.
        /// </summary>
        public void Show()
        {
            if (!this.IsVisible)
            {
                this.mintConsoleState = ConsoleState.Opening;
            }
        }

        /// <summary>
        /// Hides the console.
        /// </summary>
        public void Hide()
        {
            if (this.IsVisible)
            {
                this.mintConsoleState = ConsoleState.Closing;
            }
        }

        /// <summary>
        /// Gets/Sets the maximum size that the console can open to.
        /// </summary>
        /// <remarks>If MaxConsolePercentage is set to 100 the consle can be </remarks>
        public float MaxConsolePercentage
        {
            get
            {
                return msngMaxConsolePercentage;
            }
            set
            {
                msngMaxConsolePercentage = value;
                if (msngMaxConsolePercentage < 1)
                {
                    msngMaxConsolePercentage = 1;
                }
                if (msngMaxConsolePercentage > 100)
                {
                    msngMaxConsolePercentage = 100;
                }
                if (msngCurrentConsolePercentage > msngMaxConsolePercentage)
                {
                    msngCurrentConsolePercentage = msngMaxConsolePercentage;
                }

            }
        }

        /// <summary>
        /// Returns true if the console is visible.
        /// </summary>
        public bool IsVisible
        {
            get { return mintConsoleState != ConsoleState.Closed; }
        }

        /// <summary>
        /// Gets/Sets the percentage that opened to.
        /// </summary>
        public float CurrentConsolePercentage
        {
            get { return msngCurrentConsolePercentage; }
            set
            {
                msngCurrentConsolePercentage = value;
                if (msngCurrentConsolePercentage < 0)
                {
                    msngCurrentConsolePercentage = 0;
                }
                if (msngCurrentConsolePercentage > msngMaxConsolePercentage)
                {
                    msngCurrentConsolePercentage = msngMaxConsolePercentage;
                }
            }
        }

        /// <summary>
        /// Gets/Sets the scrolling value of the console history.
        /// </summary>
        /// <remarks>ScrollValues will be restricted between zero and the MaxScrollValues property.</remarks>
        public Vector2 ScrollValues
        {
            get { return this.scrollValues; }
            set
            {
                this.scrollValues = value;
                if (this.scrollValues.X < 0) this.scrollValues.X = 0;
                if (this.scrollValues.Y < 0) this.scrollValues.Y = 0;
                if (this.scrollValues.X > this.maxScrollValues.X) this.scrollValues.X = this.maxScrollValues.X;
                if (this.scrollValues.Y > this.maxScrollValues.Y) this.scrollValues.Y = this.maxScrollValues.Y;
            }
        }

        /// <summary>
        /// Returs the maximum value that the ScrollValues property can be set to.
        /// </summary>
        public Vector2 MaxScrollValues
        {
            get { return this.maxScrollValues; }
        }

        /// <summary>
        /// Gets/Sets weather or not to wrap the text that gets displayed in the console will be wrapped.
        /// </summary>
        public bool WordWrap
        {
            get { return this.wordWrap; }
            set { this.wordWrap = value; }
        }

        /// <summary>
        /// Gets/Sets the SpriteBatch object that will be used to draw the console text.
        /// </summary>
        public SpriteBatch Sprite
        {
            get { return sb; }
            set { sb = value; }
        }

        /// <summary>
        /// Gets/Sets the insertion point of the console entry line.
        /// </summary>
        public int InsertIndex
        {
            get { return insertIndex; }
            set
            {
                insertIndex = value;
                if (this.insertIndex < 0) this.insertIndex = 0;
                if (this.insertIndex > this.console.Entryline.Length) this.insertIndex = this.console.Entryline.Length;
            }
        }

        /// <summary>
        /// Returns the number of commands stored in the command history.
        /// </summary>
        public int CommandHistoryCount
        {
            get { return this.commandHistory.Count; }
        }

        /// <summary>
        /// Gets a entry from the command history.
        /// </summary>
        /// <param name="Index">The index of the entry to retrieve.</param>
        /// <returns>Returns a value from the command history.</returns>
        public string CommandHistory(int Index)
        {
            return this.commandHistory[Index];
        }

        /// <summary>
        /// Gets/Sets the maximum number of commands that will be stored in the command history. 
        /// </summary>
        /// <remarks>The default value is 100.</remarks>
        public int MaxCommandHistory
        {
            get { return maxCommandHistory; }
            set
            {
                maxCommandHistory = value;
                if (this.maxCommandHistory < 0) this.maxCommandHistory = 0;
                while (this.maxCommandHistory < this.commandHistory.Count)
                {
                    this.commandHistory.RemoveAt(this.commandHistory.Count - 1);
                }
                this.CommandHistoryIndex = this.historyIndex;
            }
        }


        /// <summary>
        /// Gets/Sets the current index into the command history list.
        /// </summary>
        public int CommandHistoryIndex
        {
            get { return historyIndex; }
            set
            {
                historyIndex = value;
                if (this.historyIndex < 0) this.historyIndex = 0;
                if (this.historyIndex > this.commandHistory.Count - 1) this.historyIndex = this.commandHistory.Count - 1;
            }
        }

        /// <summary>
        /// Prints an array of text out to the console.
        /// </summary>
        /// <param name="text">The text to be printed in the console.</param>
        public void Print(string[] text)
        {
            this.console.Print(text);
        }

        /// <summary>
        /// Prints a collection of strings out to the console.
        /// </summary>
        /// <param name="text">The collection of strings to be printed in the console.</param>
        public void Print(StringCollection text)
        {
            this.console.Print(text);
        }

        /// <summary>
        /// Prints text out to the console.
        /// </summary>
        /// <param name="text">The text to be printed.</param>
        public void Print(string text)
        {
            this.console.Print(text);
        }

        /// <summary>
        /// Gets/Sets the fonst to use when printing text out to the console.
        /// </summary>
        /// <remarks>Only used by the default renderer.</remarks>
        public SpriteFont Font
        {
            get { return this.font; }
            set { this.font = value; }
        }

        /// <summary>
        /// The console extender that provides aditional console functioality.
        /// </summary>
        public CommandExtenderManager ExtenderManager
        {
            get { return extenderManager; }
        }

        /// <summary>
        /// Gets/Sets the color of the console text.
        /// </summary>
        /// <remarks>This property only effects the built in console rendering system.</remarks>
        public Color TextColor
        {
            get { return textColor; }
            set { textColor = value; }
        }

        /// <summary>
        /// Gets/Sets the outside border color when using the default built in console rendering method.
        /// </summary>
        /// <remarks>This property only effects the built in console rendering system.</remarks>
        public Color BorderColor
        {
            get { return borderColor; }
            set { borderColor = value; }
        }

        /// <summary>
        /// Gets/Sets the key used to hide the console.
        /// </summary>
        /// <remarks>This key could be different from the key that is used to show the console.</remarks>
        public Keys DisplayToggle
        {
            get { return displayToggle; }
            set { displayToggle = value; }
        }

        /// <summary>
        /// Returns a reference to the command manager for the consle, that manages the consoles extensible command functionality.
        /// </summary>
        public CommandManager CommandManager
        {
            get { return commandManager; }
        }

        /// <summary>
        /// Gets/Sets the method that will be called to handle drawing the console.
        /// </summary>
        /// <remarks>The ConsoleComponent has a default console rendering delgate already assigned by default. 
        /// Set this property to null if you want to use you own method system to draw the console.</remarks>
        public RenderConsole RenderMethod
        {
            get { return rendermethod; }
            set { rendermethod = value; }
        }

        /// <summary>
        /// Returns a reference to the console object.
        /// </summary>
        public Console Console
        {
            get { return console; }
        }

        /// <summary>
        /// Gets/Sets the on screen bounds that the console will be drawn at.
        /// </summary>
        public Rectangle Bounds
        {
            get { return bounds; }
            set { bounds = value; }
        }

        /// <summary>
        /// Gets/Sets the left position that the console will be drawn at.
        /// </summary>
        public int Left
        {
            get { return bounds.X; }
            set { bounds.X = value; }
        }

        /// <summary>
        /// Gets/Sets the top position that the console will be drawn at.
        /// </summary>
        public int Top
        {
            get { return bounds.Y; }
            set { bounds.Y = value; }
        }

        /// <summary>
        /// Gets/Sets the width of the console when drawn on the screen.
        /// </summary>
        public int Width
        {
            get { return bounds.Width; }
            set { bounds.Width = value < 1 ? 1 : value; }
        }

        /// <summary>
        /// Gets/Sets the height of the console when drawn on the screen.
        /// </summary>
        public int Height
        {
            get { return bounds.Height; }
            set { bounds.Height = value < 1 ? 1 : value; }
        }

        /// <summary>
        /// Gets/Sets the background color for the console.
        /// </summary>
        /// <remarks>This property only effects the built in console rendering system.</remarks>
        public Color BackgroundColor
        {
            get { return backColor; }
            set
            {
                backColor = value;
                CreateVB(backColor);
            }
        }

        /// <summary>
        /// ConsoleComponent constructor.
        /// </summary>
        /// <param name="game">A reference to a Game object.</param>
        public ConsoleComponent(Game game)
            : base(game)
        {
            commandManager = new CommandManager();
            console = new Console(this.commandManager);
            this.rendermethod = this.Render;
            this.extenderManager = new CommandExtenderManager(this.commandManager);
            content = new ResourceContentManager(game.Services, Assets.ResourceManager);
            this.font = content.Load<SpriteFont>("CourierNew");
        }

        protected override void LoadGraphicsContent(bool loadAllContent)
        {
            base.LoadGraphicsContent(loadAllContent);
        }

        /// <summary>
        /// Restores the default renderer that is used to draw the console on the screen.
        /// </summary>
        public void RestoreDefaultRenderer()
        {
            this.rendermethod = this.Render;
            this.font = content.Load<SpriteFont>("CourierNew");
        }

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            base.Initialize();
            this.input = new UserInputService(this.Game);

            // TODO: Add your initialization code here
            CreateVB(backColor);
            if (decl == null) decl = new VertexDeclaration(this.GraphicsDevice, VertexPositionNormalTexture.VertexElements);

            if (fx != null && fx.IsDisposed == false) fx.Dispose();
            fx = null;
            fx = new BasicEffect(this.GraphicsDevice, null);

            this.sb = new SpriteBatch(this.GraphicsDevice);
            this.rendermethod = this.Render;
        }


        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            bool isScrolling = false;
            Vector2 scrollOffset = Vector2.Zero;

            if (!this.IsVisible) return;
            // update input
            this.input.Update(gameTime);

            // KeyboardState state = Keyboard.GetState();
            bool controlPressed = this.input.IsKeyDown(Keys.LeftControl) || this.input.IsKeyDown(Keys.RightControl);
            bool shiftPressed = this.input.IsKeyDown(Keys.LeftShift) || this.input.IsKeyDown(Keys.RightShift);
            if (this.input.KeyClick(Keys.CapsLock))
            {
                this.capsPressed = !this.capsPressed;
            }

            Keys[] keys = this.input.GetClickedKeys();

            // check if wanting to hide the console
            if (this.input.KeyClick(this.displayToggle))
            {
                this.Hide();
                return;
            }

            foreach (Keys k in keys)
            {
                // process the key click for the key
                switch (k)
                {

                    case Keys.Enter:
                        AddHistoryCommand();
                        console.ProcessEntryLine(true);
                        this.InsertIndex = 0;
                        this.scrollValues = Vector2.Zero;
                        break;

                    case Keys.Execute:
                        AddHistoryCommand();
                        console.ProcessEntryLine(true);
                        this.InsertIndex = 0;
                        this.scrollValues = Vector2.Zero;
                        break;

                    case Keys.Back:
                        if (controlPressed)
                        {
                            this.console.Entryline = string.Empty;
                            this.InsertIndex = 0;
                        }
                        else
                        {
                            if (this.insertIndex > 0)
                            {
                                this.console.Backspace(this.insertIndex - 1, 1);
                                this.InsertIndex--;
                            }
                        }
                        break;

                    case Keys.Delete:
                        if (this.insertIndex < this.console.Entryline.Length)
                        {
                            this.console.Backspace(this.insertIndex, 1);
                        }
                        break;

                    case Keys.Down:
                        this.CommandHistoryIndex--;
                        if (this.historyIndex != -1)
                        {
                            console.Entryline = this.commandHistory[this.historyIndex];
                        }
                        break;

                    case Keys.Home:
                        if (controlPressed)
                        {
                            this.scrollValues = this.maxScrollValues;
                            this.scrollValues.X = 0;
                        }
                        else
                        {
                            this.InsertIndex = 0;
                        }
                        break;

                    case Keys.End:
                        if (controlPressed)
                        {
                            this.scrollValues = Vector2.Zero;
                            this.scrollValues.X = 0;
                        }
                        else
                        {
                            this.InsertIndex = this.console.Entryline.Length;
                        }
                        break;

                    case Keys.Left:
                        this.InsertIndex--;
                        break;

                    case Keys.Right:
                        this.InsertIndex++;
                        break;

                    case Keys.Up:
                        this.CommandHistoryIndex++;
                        if (this.historyIndex != -1)
                        {
                            console.Entryline = this.commandHistory[this.historyIndex];
                        }
                        break;

                    case Keys.Apps:
                    case Keys.Attn:
                    case Keys.BrowserBack:
                    case Keys.BrowserFavorites:
                    case Keys.BrowserForward:
                    case Keys.BrowserHome:
                    case Keys.BrowserRefresh:
                    case Keys.BrowserSearch:
                    case Keys.BrowserStop:
                    case Keys.CapsLock:
                    //   case Keys.Delete:
                    //   case Keys.Down:
                    //    case Keys.End:
                    case Keys.EraseEof:
                    case Keys.Exsel:
                    case Keys.F1:
                    case Keys.F2:
                    case Keys.F3:
                    case Keys.F4:
                    case Keys.F5:
                    case Keys.F6:
                    case Keys.F7:
                    case Keys.F8:
                    case Keys.F9:
                    case Keys.F10:
                    case Keys.F11:
                    case Keys.F12:
                    case Keys.F13:
                    case Keys.F14:
                    case Keys.F15:
                    case Keys.F16:
                    case Keys.F17:
                    case Keys.F18:
                    case Keys.F19:
                    case Keys.F20:
                    case Keys.F21:
                    case Keys.F22:
                    case Keys.F23:
                    case Keys.F24:
                    case Keys.Help:
                    // case Keys.Home:
                    case Keys.Insert:
                    case Keys.LaunchApplication1:
                    case Keys.LaunchApplication2:
                    case Keys.LaunchMail:
                    //   case Keys.Left:
                    case Keys.LeftAlt:
                    case Keys.LeftControl:
                    case Keys.LeftShift:
                    case Keys.LeftWindows:
                    case Keys.MediaNextTrack:
                    case Keys.MediaPlayPause:
                    case Keys.MediaPreviousTrack:
                    case Keys.MediaStop:
                    case Keys.None:
                    case Keys.NumLock:
                    case Keys.Oem8:
                    case Keys.OemClear:
                    case Keys.Pa1:
                    case Keys.PageDown:
                    case Keys.PageUp:
                    case Keys.Play:
                    case Keys.Print:
                    case Keys.PrintScreen:
                    case Keys.ProcessKey:
                    //     case Keys.Right:
                    case Keys.RightAlt:
                    case Keys.RightControl:
                    case Keys.RightShift:
                    case Keys.RightWindows:
                    case Keys.Scroll:
                    case Keys.Select:
                    case Keys.SelectMedia:
                    case Keys.Sleep:
                    //  case Keys.Up:
                    case Keys.VolumeDown:
                    case Keys.VolumeMute:
                    case Keys.VolumeUp:
                    case Keys.Zoom:
                        break;

                    case Keys.Add:
                        console.PrintStringToEntryLine(this.insertIndex, "+", false, false);
                        this.InsertIndex++;
                        break;

                    case Keys.Subtract:
                        console.PrintStringToEntryLine(this.insertIndex, "-", false, false);
                        this.InsertIndex++;
                        break;

                    case Keys.Multiply:
                        console.PrintStringToEntryLine(this.insertIndex, "*", false, false);
                        this.InsertIndex++;
                        break;

                    case Keys.OemOpenBrackets:
                        if (!shiftPressed)
                        {
                            console.PrintStringToEntryLine(this.insertIndex, "[", false, false);
                        }
                        else
                        {
                            console.PrintStringToEntryLine(this.insertIndex, "{", false, false);
                        }
                        this.InsertIndex++;
                        break;

                    case Keys.OemCloseBrackets:
                        if (!shiftPressed)
                        {
                            console.PrintStringToEntryLine(this.insertIndex, "]", false, false);
                        }
                        else
                        {
                            console.PrintStringToEntryLine(this.insertIndex, "}", false, false);
                        }
                        this.InsertIndex++;
                        break;

                    case Keys.OemComma:
                        if (!shiftPressed)
                        {
                            console.PrintStringToEntryLine(this.insertIndex, ",", false, false);
                        }
                        else
                        {
                            console.PrintStringToEntryLine(this.insertIndex, "<", false, false);
                        }
                        this.InsertIndex++;
                        break;

                    case Keys.OemMinus:
                        if (!shiftPressed)
                        {
                            console.PrintStringToEntryLine(this.insertIndex, "-", false, false);
                        }
                        else
                        {
                            console.PrintStringToEntryLine(this.insertIndex, "_", false, false);
                        }
                        this.InsertIndex++;
                        break;

                    case Keys.OemPeriod:
                        if (!shiftPressed)
                        {
                            console.PrintStringToEntryLine(this.insertIndex, ".", false, false);
                        }
                        else
                        {
                            console.PrintStringToEntryLine(this.insertIndex, ">", false, false);
                        }
                        this.InsertIndex++;
                        break;

                    case Keys.OemPipe:
                        if (!shiftPressed)
                        {
                            console.PrintStringToEntryLine(this.insertIndex, @"\", false, false);
                        }
                        else
                        {
                            console.PrintStringToEntryLine(this.insertIndex, "|", false, false);
                        }
                        this.InsertIndex++;
                        break;

                    case Keys.OemPlus:
                        if (!shiftPressed)
                        {
                            console.PrintStringToEntryLine(this.insertIndex, "=", false, false);
                        }
                        else
                        {
                            console.PrintStringToEntryLine(this.insertIndex, "+", false, false);
                        }
                        this.InsertIndex++;
                        break;

                    case Keys.OemTilde:
                        if (!shiftPressed)
                        {
                            console.PrintStringToEntryLine(this.insertIndex, "`", false, false);
                        }
                        else
                        {
                            console.PrintStringToEntryLine(this.insertIndex, "~", false, false);
                        }
                        this.InsertIndex++;
                        break;

                    case Keys.Tab:
                        console.PrintStringToEntryLine(this.insertIndex, "    ", false, false);
                        this.InsertIndex += 4;
                        break;

                    case Keys.NumPad0:
                        console.PrintStringToEntryLine(this.insertIndex, "0", false, false);
                        this.InsertIndex++;
                        break;

                    case Keys.NumPad1:
                        console.PrintStringToEntryLine(this.insertIndex, "1", false, false);
                        this.InsertIndex++;
                        break;

                    case Keys.NumPad2:
                        console.PrintStringToEntryLine(this.insertIndex, "2", false, false);
                        this.InsertIndex++;
                        break;

                    case Keys.NumPad3:
                        console.PrintStringToEntryLine(this.insertIndex, "3", false, false);
                        this.InsertIndex++;
                        break;

                    case Keys.NumPad4:
                        console.PrintStringToEntryLine(this.insertIndex, "4", false, false);
                        this.InsertIndex++;
                        break;

                    case Keys.NumPad5:
                        console.PrintStringToEntryLine(this.insertIndex, "5", false, false);
                        this.InsertIndex++;
                        break;

                    case Keys.NumPad6:
                        console.PrintStringToEntryLine(this.insertIndex, "6", false, false);
                        this.InsertIndex++;
                        break;

                    case Keys.NumPad7:
                        console.PrintStringToEntryLine(this.insertIndex, "7", false, false);
                        this.InsertIndex++;
                        break;

                    case Keys.NumPad8:
                        console.PrintStringToEntryLine(this.insertIndex, "8", false, false);
                        this.InsertIndex++;
                        break;

                    case Keys.NumPad9:
                        console.PrintStringToEntryLine(this.insertIndex, "9", false, false);
                        this.InsertIndex++;
                        break;

                    case Keys.OemQuotes:
                        if (!shiftPressed)
                        {
                            console.PrintStringToEntryLine(this.insertIndex, "'", false, false);
                        }
                        else
                        {
                            console.PrintStringToEntryLine(this.insertIndex, "\"", false, false);
                        }
                        this.InsertIndex++;
                        break;

                    case Keys.OemQuestion:
                        if (!shiftPressed)
                        {
                            console.PrintStringToEntryLine(this.insertIndex, "/", false, false);
                        }
                        else
                        {
                            console.PrintStringToEntryLine(this.insertIndex, "?", false, false);
                        }
                        this.InsertIndex++;
                        break;

                    case Keys.OemBackslash:
                        if (!shiftPressed)
                        {
                            console.PrintStringToEntryLine(this.insertIndex, @"\", false, false);
                        }
                        else
                        {
                            console.PrintStringToEntryLine(this.insertIndex, "|", false, false);
                        }
                        this.InsertIndex++;
                        break;

                    case Keys.OemSemicolon:
                        if (!shiftPressed)
                        {
                            console.PrintStringToEntryLine(this.insertIndex, ";", false, false);
                        }
                        else
                        {
                            console.PrintStringToEntryLine(this.insertIndex, ":", false, false);
                        }
                        this.InsertIndex++;
                        break;

                    case Keys.Separator:
                        if (!shiftPressed)
                        {
                            console.PrintStringToEntryLine(this.insertIndex, "<", false, false);
                        }
                        else
                        {
                            console.PrintStringToEntryLine(this.insertIndex, ",", false, false);
                        }
                        this.InsertIndex++;
                        break;

                    case Keys.Divide:
                        if (!shiftPressed)
                        {
                            console.PrintStringToEntryLine(this.insertIndex, "/", false, false);
                        }
                        else
                        {
                            console.PrintStringToEntryLine(this.insertIndex, "?", false, false);
                        }
                        this.InsertIndex++;
                        break;

                    case Keys.Decimal:
                        console.PrintStringToEntryLine(this.insertIndex, ".", false, false);
                        this.InsertIndex++;
                        break;

                    case Keys.Space:
                        console.PrintStringToEntryLine(this.insertIndex, " ", false, false);
                        this.InsertIndex++;
                        break;

                    case Keys.D1:
                        if (!shiftPressed)
                        {
                            console.PrintStringToEntryLine(this.insertIndex, "1", false, false);
                        }
                        else
                        {
                            console.PrintStringToEntryLine(this.insertIndex, "!", false, false);
                        }
                        this.InsertIndex++;
                        break;

                    case Keys.D2:
                        if (!shiftPressed)
                        {
                            console.PrintStringToEntryLine(this.insertIndex, "2", false, false);
                        }
                        else
                        {
                            console.PrintStringToEntryLine(this.insertIndex, "@", false, false);
                        }
                        this.InsertIndex++;
                        break;

                    case Keys.D3:
                        if (!shiftPressed)
                        {
                            console.PrintStringToEntryLine(this.insertIndex, "3", false, false);
                        }
                        else
                        {
                            console.PrintStringToEntryLine(this.insertIndex, "#", false, false);
                        }
                        this.InsertIndex++;
                        break;

                    case Keys.D4:
                        if (!shiftPressed)
                        {
                            console.PrintStringToEntryLine(this.insertIndex, "4", false, false);
                        }
                        else
                        {
                            console.PrintStringToEntryLine(this.insertIndex, "$", false, false);
                        }
                        this.InsertIndex++;
                        break;

                    case Keys.D5:
                        if (!shiftPressed)
                        {
                            console.PrintStringToEntryLine(this.insertIndex, "5", false, false);
                        }
                        else
                        {
                            console.PrintStringToEntryLine(this.insertIndex, "%", false, false);
                        }
                        this.InsertIndex++;
                        break;

                    case Keys.D6:
                        if (!shiftPressed)
                        {
                            console.PrintStringToEntryLine(this.insertIndex, "6", false, false);
                        }
                        else
                        {
                            console.PrintStringToEntryLine(this.insertIndex, "^", false, false);
                        }
                        this.InsertIndex++;
                        break;

                    case Keys.D7:
                        if (!shiftPressed)
                        {
                            console.PrintStringToEntryLine(this.insertIndex, "7", false, false);
                        }
                        else
                        {
                            console.PrintStringToEntryLine(this.insertIndex, "&", false, false);
                        }
                        this.InsertIndex++;
                        break;

                    case Keys.D8:
                        if (!shiftPressed)
                        {
                            console.PrintStringToEntryLine(this.insertIndex, "8", false, false);
                        }
                        else
                        {
                            console.PrintStringToEntryLine(this.insertIndex, "*", false, false);
                        }
                        this.InsertIndex++;
                        break;

                    case Keys.D9:
                        if (!shiftPressed)
                        {
                            console.PrintStringToEntryLine(this.insertIndex, "9", false, false);
                        }
                        else
                        {
                            console.PrintStringToEntryLine(this.insertIndex, "(", false, false);
                        }
                        this.InsertIndex++;
                        break;

                    case Keys.D0:
                        if (!shiftPressed)
                        {
                            console.PrintStringToEntryLine(this.insertIndex, "0", false, false);
                        }
                        else
                        {
                            console.PrintStringToEntryLine(this.insertIndex, ")", false, false);
                        }
                        this.InsertIndex++;
                        break;

                    default:
                        string c;
                        c = k.ToString();
                        // check to convert the character that was typed to lower case
                        if (!capsPressed & !shiftPressed)
                        {
                            c = c.ToLower();
                        }
                        console.PrintStringToEntryLine(this.insertIndex, c, false, false);

                        this.InsertIndex++;
                        break;
                }


            } // end of for loop

            // ============================ 
            // check if the user is trying to scroll
            // ============================ 
            if (controlPressed && this.input.IsKeyDown(Keys.Down))
            {
                scrollOffset -= Vector2.UnitY;
                isScrolling = true;
                this.input.SetKeyState(Keys.Down, KeyState.Up);
                this.input.SetPreviuosKeyState(Keys.Down, KeyState.Up);
            }

            if (controlPressed && this.input.IsKeyDown(Keys.Left))
            {
                scrollOffset -= Vector2.UnitX;
                isScrolling = true;
                this.input.SetKeyState(Keys.Left, KeyState.Up);
                this.input.SetPreviuosKeyState(Keys.Left, KeyState.Up);
            }

            if (controlPressed && this.input.IsKeyDown(Keys.Right))
            {
                scrollOffset += Vector2.UnitX;
                isScrolling = true;
                this.input.SetKeyState(Keys.Right, KeyState.Up);
                this.input.SetPreviuosKeyState(Keys.Right, KeyState.Up);
            }

            if (controlPressed && this.input.IsKeyDown(Keys.Up))
            {
                scrollOffset += Vector2.UnitY;
                isScrolling = true;
                this.input.SetKeyState(Keys.Up, KeyState.Up);
                this.input.SetPreviuosKeyState(Keys.Up, KeyState.Up);
            }

            if (this.input.IsKeyDown(Keys.PageUp))
            {
                scrollOffset += Vector2.UnitY * 3;
                isScrolling = true;
            }

            if (this.input.IsKeyDown(Keys.PageDown))
            {
                scrollOffset -= Vector2.UnitY * 3;
                isScrolling = true;
            }

            // handles game pad input
            HandleGamepadInput(ref isScrolling, ref scrollOffset, gameTime);

            // scroll the entries if the user is requesting it
            if (isScrolling)
            {
                this.ScrollValues += scrollOffset * gameTime.ElapsedGameTime.Milliseconds;
            }
        }

        private void HandleGamepadInput(ref bool isScrolling, ref Vector2 scrollOffset, GameTime gameTime)
        {
            // check if game pad input is allowed
            if (this.allowGamepadControl)
            {
                // check if joystick is scrolling
                // =============================
                if (this.input.RightThumbStickChanged(this.gamepadControl))
                {
                    scrollOffset += this.input.RightThumbStick(this.gamepadControl);
                    isScrolling = true;
                }

                // check if wanting to backspace
                if (this.input.BClick(this.gamepadControl))
                {
                    // check if wanting to add spaces
                    if (this.input.Y(this.gamepadControl) == ButtonState.Pressed)
                    {
                        this.console.Backspace(this.insertIndex , 1);
                    }
                    else
                    {
                        this.console.Backspace(this.insertIndex - 1, 1);
                        this.InsertIndex--;
                    }
                }

                // check if wanting to backspace
                if (this.input.AClick(this.gamepadControl))
                {
                    string line = this.console.Entryline;
                    if (line != string.Empty)
                    {
                        line = line.Insert(this.insertIndex, line[this.insertIndex].ToString());
                        this.console.Entryline = line;
                        this.InsertIndex++;
                    }
                }

                // convert char to upper or lower case
                if (this.input.XClick(this.gamepadControl))
                {
                    string line = this.console.Entryline;
                    if (line != string.Empty)
                    {
                        char c = line[this.insertIndex];
                        if (char.IsLetter(c) && char.IsLower(c))
                        {
                            c = c.ToString().ToUpper()[0];
                        }
                        else if (char.IsLetter(c) && char.IsUpper(c))
                        {
                            c = c.ToString().ToLower()[0];
                        }
                        line = line.Remove(this.insertIndex, 1);
                        line = line.Insert(this.insertIndex, c.ToString());
                        this.console.Entryline = line;
                    }
                }

                // check if positioning the insertion point
                if (this.input.LeftClick(this.gamepadControl))
                {
                    string line = this.console.Entryline;
                    if (this.insertIndex == line.Length && line != string.Empty && line.Substring(line.Length - 1, 1) == " ")
                    {
                        this.console.Backspace(this.insertIndex - 1, 1);
                    }
                    this.InsertIndex--;
                }

                if (this.input.RightClick(this.gamepadControl))
                {
                    // check if wanting to add spaces
                    if (this.input.Y(this.gamepadControl) == ButtonState.Pressed)
                    {
                        this.console.PrintStringToEntryLine(this.insertIndex, " ");
                        this.InsertIndex++;
                    }
                    else // not  trying to insert spaces so reposition insertion point
                    {
                        if (this.insertIndex == this.console.Entryline.Length)
                        {
                            this.console.PrintStringToEntryLine(" ", false, false);
                        }
                        this.InsertIndex++;
                    }

                }

                // check if changing the character at the insertion point
                if (this.input.UpClick(this.gamepadControl))
                {
                    // check if navigating the comamnd history
                    if (this.input.Y(this.gamepadControl) == ButtonState.Pressed)
                    {
                        this.CommandHistoryIndex++;
                        if (this.historyIndex != -1)
                        {
                            console.Entryline = this.commandHistory[this.historyIndex];
                        }
                    }
                    else // not navigating command history so change character
                    {
                        ChangeChar(-1, 127);
                    }
                }

                if (this.input.DownClick(this.gamepadControl))
                {
                    // check if navigating the comamnd history
                    if (this.input.Y(this.gamepadControl) == ButtonState.Pressed)
                    {
                        this.CommandHistoryIndex--;
                        if (this.historyIndex != -1)
                        {
                            console.Entryline = this.commandHistory[this.historyIndex];
                        }
                    }
                    else // not navigating command history so change character
                    {
                        ChangeChar(1, 31);
                    }
                }

                // check if clearing the entry line
                if (this.input.LeftStickClick(this.gamepadControl))
                {
                    this.console.Entryline = string.Empty;
                    this.InsertIndex = 0;
                }

                // check if moving to begining of entry line 
                if (this.input.LeftSholderClick(this.gamepadControl))
                {
                    this.InsertIndex = 0;
                }

                // check if moving to end of entry line 
                if (this.input.RightSholderClick(this.gamepadControl))
                {
                    this.InsertIndex = this.console.Entryline.Length;
                }

                // check if hiding the console
                if (this.input.BackClick(this.gamepadControl))
                {
                    this.Hide();
                }

                // check if changing character using the left trigger button
                // flutter the trigger to change the character
                if (this.input.LeftTriggerChanged(this.gamepadControl))
                {
                    float value = this.input.LeftTrigger(this.gamepadControl) * gameTime.ElapsedGameTime.Milliseconds;
                    value = MathHelper.Clamp(value, 0, 1);
                    ChangeChar((int)-value, 127);
                }

                // check if changing character using the right trigger button
                // flutter the trigger to change the character
                if (this.input.RightTriggerChanged(this.gamepadControl))
                {
                    float value = this.input.RightTrigger(this.gamepadControl) * gameTime.ElapsedGameTime.Milliseconds;
                    value = MathHelper.Clamp(value, 0, 1);
                    ChangeChar((int)value, 31);
                }

                // press start to execure the entry line
                if (this.input.StartClick(this.gamepadControl))
                {
                    AddHistoryCommand();
                    console.ProcessEntryLine(true);
                    this.InsertIndex = 0;
                    this.scrollValues = Vector2.Zero;
                }
            }
        }

        private void ChangeChar(int value, int startvalue)
        {
            string line = this.console.Entryline;
            //try
            //{
            //    line = line.Substring(this.insertIndex, 1);
            //}
            //catch
            //{
            //    if (this.insertIndex == line.Length)
            //    {
            //        line = Convert.ToChar((int)MathHelper.Clamp(startvalue, 32, 126)).ToString();
            //    }
            //}
            if (this.insertIndex >= line.Length)
            {
                line += new string(' ', this.insertIndex - line.Length < 1 ? 1 : this.insertIndex - line.Length);
                //this.InsertIndex--;
            }

            int ord = Convert.ToInt32(line[this.insertIndex]);
            ord = (int)MathHelper.Clamp(ord + value, 32, 126);
            try
            {
                line = line.Remove(this.insertIndex, 1);
                this.console.Entryline = line.Insert(this.insertIndex, Convert.ToChar(ord).ToString());
            }
            catch { }
        }

        private void AddHistoryCommand()
        {
            if (this.commandHistory.Contains(this.console.Entryline))
            {
                this.commandHistory.Remove(this.console.Entryline);
            }
            this.commandHistory.Insert(0, this.console.Entryline);
            if (this.commandHistory.Count > this.maxCommandHistory)
            {
                this.commandHistory.RemoveAt(this.commandHistory.Count - 1);
            }
            this.historyIndex = 0;
        }

        /// <summary>
        /// Allows the game component to draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Draw(GameTime gameTime)
        {

            this.Render(gameTime);
            //if (this.IsVisible == false)
            //{
            //    this.Visible = false;
            //    return;
            //}

            base.Draw(gameTime);
        }

        private void DoRender(GameTime gameTime)
        {
            Properties.Settings sett = Properties.Settings.Default;

            if (gameTime.TotalGameTime.Ticks > RenderConsole_LastCursonBlinkTime + TimeSpan.FromMilliseconds(sett.CursorBlinkRate).Ticks)
            {
                RenderConsole_CursorVisible = !RenderConsole_CursorVisible;
                RenderConsole_LastCursonBlinkTime = gameTime.TotalGameTime.Ticks;
            }

            int consoleHeight = (int)(this.bounds.Height * (this.MaxConsolePercentage / 100));
            int visibleHeight = (int)(this.bounds.Height * (this.CurrentConsolePercentage / 100));

            int x = this.bounds.X;
            int y = this.bounds.Y;
            int line = y + (visibleHeight - this.font.LineSpacing);
            int width = this.bounds.Width;
            Rectangle consoleBackground = new Rectangle(x, y, width, visibleHeight);

            DrawRect(consoleBackground, true, this.backColor);

            DrawRect(consoleBackground, false, this.borderColor);
            if (this.font == null) return;

            // hold the current state of the sissor rect
            Rectangle sr = this.GraphicsDevice.ScissorRectangle;
            // set the sissor rect to the console dimentions so text does not get drawn out of bounds
            Rectangle nsr;
            nsr = Helpers.RectangleIntersect(Helpers.ViewportRectangle(this.GraphicsDevice.Viewport), consoleBackground);
            if (nsr == Rectangle.Empty)
            {
                this.GraphicsDevice.ScissorRectangle = consoleBackground;
            }
            else
            {
                this.GraphicsDevice.ScissorRectangle = nsr;
            }

            // hold the state of the sissor test
            bool sTest = this.GraphicsDevice.RenderState.ScissorTestEnable;
            // enable sissor testing
            this.GraphicsDevice.RenderState.ScissorTestEnable = true;

            DrawRect(new Rectangle(consoleBackground.Left, consoleBackground.Bottom - this.font.LineSpacing,
            consoleBackground.Width, 0), false, this.borderColor);

            consoleBackground.Inflate(-4, 0);
            // draw entry line text with the blinking cursor at the end
            string startEntryLineText = string.Empty;
            string entryLineText = this.console.Entryline;
            if (this.insertIndex > entryLineText.Length)
            {
                startEntryLineText = entryLineText;
            }
            else
            {
                if (this.insertIndex != 0) startEntryLineText = this.console.Entryline.Substring(0, this.insertIndex);
            }

            this.sb.Begin(SpriteBlendMode.Additive, SpriteSortMode.Immediate, SaveStateMode.SaveState);
            Vector2 totalSize = this.font.MeasureString(entryLineText);
            float entrylineOffest = 0;

            // this is a simple hack for really long test that gets typed into the entry line
            // we will have it scroll left and right depending on where the user has places the cursor
            // NOTE: the behavior is inconsistant with the standard windows text box behavior and should be 
            // modified to match the tetboxes behavior, but it is good enoughfor right now.
            if (totalSize.X < consoleBackground.Width)
            {
                entrylineOffest = 0;
            }
            else
            {
                float centerOfEntryLine = consoleBackground.Left + (consoleBackground.Width / 2);
                Vector2 size = this.font.MeasureString(entryLineText.Substring(0, this.insertIndex));
                float cursorLocation = consoleBackground.Left + size.X;
                entrylineOffest = cursorLocation - centerOfEntryLine;
                if (entrylineOffest < 0) entrylineOffest = 0;
            }


            // draw the text for the entry line
            this.sb.DrawString(this.font, entryLineText, new Vector2(consoleBackground.Left - entrylineOffest, line), this.textColor);

            // draw the blinking cursor if it is visible
            if (RenderConsole_CursorVisible)
            {
                float xcurpos = this.font.MeasureString(startEntryLineText).X;
                this.sb.DrawString(this.font, sett.CursorBlinkChar.ToString(),
                                new Vector2(consoleBackground.Left + xcurpos - entrylineOffest, line),
                                this.textColor);

            }

            // chop off the height of the entry line text
            consoleBackground.Height -= this.font.LineSpacing;
            this.sb.End();
            this.GraphicsDevice.ScissorRectangle = consoleBackground;
            this.sb.Begin(SpriteBlendMode.Additive, SpriteSortMode.Immediate, SaveStateMode.SaveState);


            // draw stored entries from the history
            Rectangle textBounds = consoleBackground;
            Vector2 tmpScroll;
            tmpScroll.X = -this.scrollValues.X;
            tmpScroll.Y = this.scrollValues.Y;
            Helpers.DrawString(this.sb, this.font, this.console.History, consoleBackground, this.textColor, this.wordWrap, tmpScroll, out textBounds);

            // calc the maximum scroll value
            this.maxScrollValues = new Vector2(textBounds.Width - consoleBackground.Width, textBounds.Height - consoleBackground.Height);
            // cap the maximum scroll values
            if (this.maxScrollValues.X < 0) this.maxScrollValues.X = 0;
            if (this.maxScrollValues.Y < 0) this.maxScrollValues.Y = 0;
            // cap the scroll values
            if (this.scrollValues.X < 0) this.scrollValues.X = 0;
            if (this.scrollValues.Y < 0) this.scrollValues.Y = 0;
            if (this.scrollValues.X > this.maxScrollValues.X) this.scrollValues.X = this.maxScrollValues.X;
            if (this.scrollValues.Y > this.maxScrollValues.Y) this.scrollValues.Y = this.maxScrollValues.Y;

            this.sb.End();

            // restore sissor rect and test
            this.GraphicsDevice.ScissorRectangle = sr;
            this.GraphicsDevice.RenderState.ScissorTestEnable = sTest;
        }

        private void DrawRect(Rectangle r, bool fill, Color col)
        {
            // setup orthographic projection with same bounds as graphics devices viewport
            fx.Projection = Matrix.CreateOrthographicOffCenter(this.GraphicsDevice.Viewport.X,
                                                               this.GraphicsDevice.Viewport.X + this.GraphicsDevice.Viewport.Width,
                                                               this.GraphicsDevice.Viewport.Y + this.GraphicsDevice.Viewport.Height,
                                                               this.GraphicsDevice.Viewport.Y, 1, 10000);
            fx.View = Matrix.CreateLookAt(Vector3.Backward, Vector3.Zero, Vector3.Up);
            fx.World = Matrix.CreateScale(r.Width, r.Height, 1);
            fx.World *= Matrix.CreateTranslation(r.X, r.Y, 0);
            fx.VertexColorEnabled = false;
            fx.TextureEnabled = false;
            fx.LightingEnabled = false;
            fx.DiffuseColor = col.ToVector3();
            fx.Alpha = col.A / (float)byte.MaxValue;
            fx.Texture = null;
            fx.Begin();
            for (int idx = 0; idx <= fx.CurrentTechnique.Passes.Count - 1; idx++)
            {
                EffectPass pass;
                pass = fx.CurrentTechnique.Passes[idx];
                pass.Begin();
                this.GraphicsDevice.VertexDeclaration = decl;
                this.GraphicsDevice.Vertices[0].SetSource(vb, 0, VertexPositionNormalTexture.SizeInBytes);
                if (fill)
                {
                    this.GraphicsDevice.RenderState.CullMode = CullMode.CullClockwiseFace;
                    this.GraphicsDevice.DrawPrimitives(PrimitiveType.TriangleFan, 0, 2);
                }
                else
                {
                    this.GraphicsDevice.DrawPrimitives(PrimitiveType.LineStrip, 0, 4);
                }
                pass.End();
            }
            fx.End();
        }

        /// <summary>
        /// Creates a triangle fan of 2 polys along the x/y axis starting at 0,0,0 with a width and height value of one.
        /// </summary>
        /// <returns>Returns a new vertexbuffer.</returns>
        private void CreateVB(Color col)
        {
            VertexPositionNormalTexture[] verts = new VertexPositionNormalTexture[5];
            verts[0] = new VertexPositionNormalTexture(new Vector3(0, 0, 0), new Vector3(0, 0, 0) + Vector3.Backward, Vector2.Zero);
            verts[1] = new VertexPositionNormalTexture(new Vector3(0, 1, 0), new Vector3(0, 1, 0) + Vector3.Backward, Vector2.UnitY);
            verts[2] = new VertexPositionNormalTexture(new Vector3(1, 1, 0), new Vector3(1, 1, 0) + Vector3.Backward, Vector2.UnitX + Vector2.UnitY);
            verts[3] = new VertexPositionNormalTexture(new Vector3(1, 0, 0), new Vector3(1, 0, 0) + Vector3.Backward, Vector2.UnitX);
            verts[4] = verts[0];

            if (vb != null && vb.IsDisposed == false) vb.Dispose();
            vb = null;
            vb = new VertexBuffer(this.GraphicsDevice, verts.Length * VertexPositionNormalTexture.SizeInBytes,
                                  ResourceUsage.None, ResourceManagementMode.Automatic);
            vb.SetData<VertexPositionNormalTexture>(verts);
            return;
        }

    }
}


