/*
Created by: X
http://www.createdbyx.com/

xGameConsoleXNA is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
using System.Diagnostics;
//using System.Data;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace xGameConsoleXNA
{
    /// <summary>
    /// Contains helper methods for working with the console.
    /// </summary>
    public sealed class Helpers
    {

        /// <summary>
        /// Checks if a string is a valid name.
        /// </summary>
        /// <param name="varName">The name to be checked.</param>
        /// <returns>Returns true if the name is valid.</returns>
        /// <remarks>IsValidName can be used to check for invalid command or alias names. 
        /// A valid name is a name that only contains letters, numbers or underscores. The same as declaring varibles in C# or vb.net.</remarks>
        /// <example>
        /// Valid names<br/>
        /// _Foo<br/>
        /// Foo123<br/>
        /// Foo_123<br/>
        /// Invalid names<br/>
        /// 123Foo<br/>
        /// Foo+123<br/>
        /// Foo?123<br/>
        /// </example>
        public static bool IsValidName(string varName)
        {
            if (varName == null || varName == string.Empty) return false;
            if (varName.IndexOfAny("!@#$%^&*()[]{}|'\"';:<>,./?`~+=- ".ToCharArray()) != -1) return false; // check for bad chars
            if (varName[0].ToString().IndexOfAny("0123456789".ToCharArray()) != -1) return false; // check if availible name starts with a number
            // build a list of invalid chars
            string tmp = string.Empty;
            for (int idx = 0; idx < 32; idx++) { tmp += Convert.ToChar(idx); }
            for (int idx = 123; idx < 256; idx++) { tmp += Convert.ToChar(idx); }
            // check for invalid chars
            if (varName.IndexOfAny(tmp.ToCharArray()) != -1) return false;

            return true;
        }

        internal static void DrawString(SpriteBatch sb, SpriteFont fnt, StringCollection text, Rectangle r, Color col)
        {
            Rectangle tmp;
            Helpers.DrawString(sb, fnt, text, r, col, true, Vector2.Zero, out tmp);
        }

        internal static void DrawString(SpriteBatch sb, SpriteFont fnt, StringCollection text, Rectangle r,
            Color col, bool performWordWrap)
        {
            Rectangle tmp;
            Helpers.DrawString(sb, fnt, text, r, col, performWordWrap, Vector2.Zero, out tmp);
        }

        internal static void DrawString(SpriteBatch sb, SpriteFont fnt, StringCollection text, Rectangle r,
                 Color col, bool performWordWrap, out Rectangle textBounds)
        {
            Helpers.DrawString(sb, fnt, text, r, col, performWordWrap, Vector2.Zero, out textBounds);
        }

        internal static void DrawString(SpriteBatch sb, SpriteFont fnt, StringCollection text, Rectangle r,
                        Color col, bool performWordWrap, Vector2 offsett, out Rectangle textBounds)
        {

            // check if there is text to draw
            textBounds = r;
            if (text == null && text.Count == 0) return;
            StringCollection lines = new StringCollection();

            // duplicate the collection
            lock (text.SyncRoot)
            {
                for (int idx = 0; idx < text.Count; idx++)
                {
                    lines.Add(text[idx]);
                }
            }

            Rectangle tmprect = Helpers.ProcessLines(fnt, r, performWordWrap, lines);


            // draw each line of text
            Vector2 pos = new Vector2(r.X, r.Bottom - tmprect.Height);

            for (int idx = 0; idx < lines.Count; idx++)
            {
                // if we are going to start drawing below the rectangles boundary then we can exit
                // if (r.Bottom - pos.Y < fnt.LineSpacing) break;
                // draw the line of text
                sb.DrawString(fnt, lines[idx], pos + offsett, col);
                // move to the next position to begin drawing the next line of text
                pos.Y += fnt.LineSpacing;
                // check if the text is even going to be drawn within the rect
                if (pos.Y > r.Bottom) break;
            }

            textBounds = tmprect;

        }

        internal static Rectangle ProcessLines(SpriteFont fnt, Rectangle r, bool performWordWrap, StringCollection lines)
        {
            // llop through each line in the collection
            Rectangle bounds = r;
            bounds.Width = 0;
            bounds.Height = 0;
            int index = 0;
            float Width = 0;
            bool lineInserted = false;
            while (index < lines.Count)
            {
                // get a line of text
                string linetext = lines[index];
                //measure the line of text
                Vector2 size = fnt.MeasureString(linetext);

                // check if the line of text is geater then then the rect we want to draw it inside of
                if (performWordWrap && size.X > r.Width)
                {
                    // find last space character in line
                    string endspace = string.Empty;
                    // deal with trailing spaces
                    if (linetext.EndsWith(" "))
                    {
                        endspace = " ";
                        linetext = linetext.TrimEnd();
                    }

                    // get the index of the last space character
                    int i = linetext.LastIndexOf(" ");
                    if (i != -1)
                    {
                        // if there was a space grab the last word in the line
                        string lastword = linetext.Substring(i + 1);
                        // move word to next line 
                        if (index == lines.Count - 1)
                        {
                            lines.Add(lastword);
                            lineInserted = true;
                        }
                        else
                        {
                            // prepend last word to begining of next line
                            if (lineInserted)
                            {
                                lines[index + 1] = lastword + endspace + lines[index + 1];
                            }
                            else
                            {
                                lines.Insert(index + 1, lastword);
                                lineInserted = true;
                            }
                        }

                        // crop last word from the line that is being processed
                        lines[index] = linetext.Substring(0, i + 1);

                    }
                    else
                    {
                        // there appear to be no space characters on this line s move to the next line
                        lineInserted = false;
                        size = fnt.MeasureString(lines[index]);
                        if (size.X > bounds.Width) Width = size.X;
                        bounds.Height += fnt.LineSpacing;// size.Y - 1;
                        index++;
                    }
                }
                else
                {
                    // this line will fit so we can skip to the next line
                    lineInserted = false;
                    size = fnt.MeasureString(lines[index]);
                    if (size.X > bounds.Width) bounds.Width = (int)size.X;
                    bounds.Height += fnt.LineSpacing;//size.Y - 1;
                    index++;
                }
            }
            // bounds.Height = (int)Height;
            return bounds;
        }

        /// <summary>
        /// Gets an attribute from a method.
        /// </summary>
        /// <typeparam name="T">A generic type where T inherits from Attribute.</typeparam>
        /// <param name="m">The member info of the member to be checked.</param>
        /// <returns>Returns the desired attribute if found else returns null (Nothing in vb.net).</returns>
        public static T GetAttribute<T>(System.Reflection.MemberInfo m) where T : Attribute
        {
            // get command attributes
            object[] items = m.GetCustomAttributes(false);
            foreach (object att in items)
            {
                // check if the VisibleToUsers was specified
                if (att is T) return (T)att;
            }

            return null;
        }


        //public static int FindLongestString(string[] items)
        //{
        //    int count = 0;
        //    foreach (string s in items)
        //    {
        //        if (count < s.Length) count = s.Length;
        //    }
        //    return count;
        //}


        /// <summary>
        /// Returns the intersection of two rectangles.
        /// </summary>
        /// <param name="r1">First rectangle.</param>
        /// <param name="r2">Second rectangle.</param>
        /// <returns>Returns a rectangle that represents the intersection of the two specified rectangles.</returns>
        /// <remarks> This rectangle intersect code brout to you by Mono! ... Well sort of...
        /// Code was edited to include IntersectsWithInclusive and FromLTRB methods code
        /// Get the Modo Code here http://go-mono.com/sources-stable/</remarks>
        public static Rectangle RectangleIntersect(Rectangle r1, Rectangle r2)
        {
            // MS.NET returns a non-empty rectangle if the two rectangles
            // touch each other
            if (!((r1.Left > r2.Right) || (r1.Right < r2.Left) ||
                 (r1.Top > r2.Bottom) || (r1.Bottom < r2.Top)))
                return Rectangle.Empty;

            int L, T, R, B;
            L = Math.Max(r1.Left, r2.Left);
            T = Math.Max(r1.Top, r2.Top);
            R = Math.Min(r1.Right, r2.Right);
            B = Math.Min(r1.Bottom, r2.Bottom);
            return new Rectangle(L, T, R - L, B - T);
        }

        /// <summary>
        /// Constructs a rectangled based on a viewport.
        /// </summary>
        /// <param name="vp">A viewport that defies the dimentions of the rectangle that will be returned.</param>
        /// <returns>Returns a rectangle with the dimentions of the viewport.</returns>
        public static Rectangle ViewportRectangle(Viewport vp)
        {
            return new Rectangle(vp.X, vp.Y, vp.Width, vp.Height);
        }

        /// <summary>
        /// Determins if a string is numeric.
        /// </summary>
        /// <param name="value">The string to check.</param>
        /// <returns>Returns true if the string starts with a numerical value.</returns>
        public static bool IsNumeric(string value)
        {
            value = value.Trim();
            if (value == string.Empty) return false;
            if (char.IsSymbol(value[0]) == false && char.IsNumber(value[0]) == false) return false;
            if (char.IsSymbol(value[0]) && value.Length > 1 && char.IsNumber(value[1]) == false) return false;
            return true;
        }

        /// <summary>
        /// Converts a StringCollection into a string.
        /// </summary>
        /// <param name="Values">The collection to be converted.</param>
        /// <param name="StartIndex">The starting index where conversion begins.</param>
        /// <param name="Separator">The string that will be used as padding between entries.</param>
        /// <returns>Returns a string of the combined entries.</returns>
        public static string StringArrayToString(StringCollection Values, int StartIndex, char Separator)
        {
            return StringArrayToString(Values, StartIndex, Values.Count, Separator);
        }

        /// <summary>
        /// Converts a StringCollection into a string.
        /// </summary>
        /// <param name="Values">The collection to be converted.</param>
        /// <param name="StartIndex">The starting index where conversion begins.</param>
        /// <param name="Count">The number of entries to convert.</param>
        /// <param name="Separator">The string that will be used as padding between entries.</param>
        /// <returns>Returns a string of the combined entries.</returns>
        public static string StringArrayToString(StringCollection Values, int StartIndex, int Count, char Separator)
        {
            if (Values == null) return string.Empty;
            if (Values.Count == 0) return string.Empty;

            string Tmp = string.Empty;

            for (int idx = StartIndex; idx <= StartIndex + Count - 1; idx++)
            {
                if (idx > Values.Count - 1) break;
                Tmp += Separator + Values[idx];
            }

            Tmp = Tmp.TrimStart(Separator);
            return Tmp;
        }

        /// <summary>
        /// Converts a StringCollection into a string.
        /// </summary>
        /// <param name="Values">The collection to be converted.</param>
        /// <param name="Separator">The string that will be used as padding between entries.</param>
        /// <returns>Returns a string of the combined entries.</returns>
        public static string StringCollectionToString(StringCollection Values, string Separator)
        {
            if (Values == null) return string.Empty;
            string[] TmpArgs = null;
            TmpArgs = new string[Values.Count];
            Values.CopyTo(TmpArgs, 0);
            return string.Join(Separator, TmpArgs);
        }

        /// <summary>
        /// Converts a StringCollection into a string.
        /// </summary>
        /// <param name="Values">The collection to be converted.</param>
        public static string[] StringCollectionToArray(StringCollection Values)
        {
            string[] TmpArgs = null;
            TmpArgs = new string[Values.Count];
            Values.CopyTo(TmpArgs, 0);
            return TmpArgs;
        }

        /// <summary>
        /// Converts a string array into a string.
        /// </summary>
        /// <param name="Values">The array to be converted.</param>
        /// <param name="StartIndex">The starting index where conversion begins.</param>
        /// <param name="Count">The number of entries to convert.</param>
        /// <param name="Separator">The string that will be used as padding between entries.</param>
        /// <returns>Returns a string of the combined entries.</returns>
        public static string StringArrayToString(string[] Values, int StartIndex, int Count, string Separator)
        {
            if (Values == null) return string.Empty;
            if (Values.Length == 0) return string.Empty;

            string Tmp = string.Empty;

            for (int idx = StartIndex; idx <= StartIndex + Count - 1; idx++)
            {
                if (idx > Values.Length - 1) break;
                Tmp += Separator + Values[idx];
            }

            Tmp = Tmp.TrimStart(Separator.ToCharArray());
            return Tmp;
        }

        /// <summary>
        /// Converts a string array into a string.
        /// </summary>
        /// <param name="Values">The array to be converted.</param>
        /// <param name="StartIndex">The starting index where conversion begins.</param>
        /// <param name="Separator">The string that will be used as padding between entries.</param>
        /// <returns>Returns a string of the combined entries.</returns>
        public static string StringArrayToString(string[] Values, int StartIndex, string Separator)
        {
            return StringArrayToString(Values, StartIndex, Values.Length, Separator);
        }
    }

}
