#define Win32
/*
Created by: X
http://www.createdbyx.com/

xGameConsoleXNA is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;


namespace xGameConsoleXNA
{
    /// <summary>
    /// Provides an game component for accessing Keyboard mouse and gamepad info.
    /// </summary>
    public class UserInputService : GameComponent, IUserInputService
    {
        private struct MouserState
        {
            public ButtonState LeftButton;
            public ButtonState RightButton;
            public ButtonState MiddleButton;
            public ButtonState XButton1;
            public ButtonState XButton2;
            public int X;
            public int Y;
            public int ScrollWheelValue;

            public Vector2 Position
            {
                get { return new Vector2(this.X, this.Y); }
                set
                {
                    this.X = (int)value.X;
                    this.Y = (int)value.Y;
                }
            }

        }

        private class KeybeardState : ICloneable
        {
            public Dictionary<Keys, KeyState> keys;

            public KeybeardState()
            {
                this.keys = new Dictionary<Keys, KeyState>();
                Array items = Enum.GetValues(typeof(Keys));
                foreach (object k in items)
                {
                    this.keys.Add((Keys)k, KeyState.Up);
                }
            }

            public bool IsKeyDown(Keys key)
            {
                return this.keys[key] == KeyState.Down;
            }

            public bool IsKeyUp(Keys key)
            {
                return this.keys[key] == KeyState.Up;
            }

            public KeybeardState(Keys[] pressedkeys)
                : this()
            {
                foreach (Keys k in pressedkeys)
                {
                    this.keys[k] = KeyState.Down;
                }
            }

            public Keys[] GetPressedKeys()
            {
                List<Keys> items = new List<Keys>();
                foreach (Keys k in this.keys.Keys)
                {
                    if (this.keys[k] == KeyState.Down) items.Add(k);
                }

                return items.ToArray();
            }

            public Keys[] GetReleasedKeys()
            {
                List<Keys> items = new List<Keys>();
                foreach (Keys k in this.keys.Keys)
                {
                    if (this.keys[k] == KeyState.Up) items.Add(k);
                }

                return items.ToArray();
            }
            #region ICloneable Members

            public object Clone()
            {
                KeybeardState kb = new KeybeardState();
                foreach (Keys k in this.keys.Keys)
                {
                    kb.keys[k] = this.keys[k];
                }

                return kb;
            }

            #endregion
        }

        #region Fields
        private ButtonState[] _A = new ButtonState[4];
        private ButtonState[] _B = new ButtonState[4];
        private ButtonState[] _X = new ButtonState[4];
        private ButtonState[] _Y = new ButtonState[4];
        private ButtonState[] _Back = new ButtonState[4];
        private ButtonState[] _Start = new ButtonState[4];
        private ButtonState[] _LeftSholder = new ButtonState[4];
        private ButtonState[] _LeftStick = new ButtonState[4];
        private ButtonState[] _RightSholder = new ButtonState[4];
        private ButtonState[] _RightStick = new ButtonState[4];
        private ButtonState[] _Left = new ButtonState[4];
        private ButtonState[] _Right = new ButtonState[4];
        private ButtonState[] _Up = new ButtonState[4];
        private ButtonState[] _Down = new ButtonState[4];
        private Vector2[] _LeftThumbStick = new Vector2[4];
        private Vector2[] _RightThumbStick = new Vector2[4];
        private float[] _LeftTrigger = new float[4];
        private float[] _RightTrigger = new float[4];
        private bool[] _IsConnected = new bool[4];
        private KeybeardState _kbState = new KeybeardState();
        private MouserState _mouseState = new MouserState();

        private ButtonState[] _PreviousA = new ButtonState[4];
        private ButtonState[] _PreviousB = new ButtonState[4];
        private ButtonState[] _PreviousX = new ButtonState[4];
        private ButtonState[] _PreviousY = new ButtonState[4];
        private ButtonState[] _PreviousBack = new ButtonState[4];
        private ButtonState[] _PreviousStart = new ButtonState[4];
        private ButtonState[] _PreviousLeftSholder = new ButtonState[4];
        private ButtonState[] _PreviousLeftStick = new ButtonState[4];
        private ButtonState[] _PreviousRightSholder = new ButtonState[4];
        private ButtonState[] _PreviousRightStick = new ButtonState[4];
        private ButtonState[] _PreviousLeft = new ButtonState[4];
        private ButtonState[] _PreviousRight = new ButtonState[4];
        private ButtonState[] _PreviousUp = new ButtonState[4];
        private ButtonState[] _PreviousDown = new ButtonState[4];
        private Vector2[] _PreviousLeftThumbStick = new Vector2[4];
        private Vector2[] _PreviousRightThumbStick = new Vector2[4];
        private float[] _PreviousLeftTrigger = new float[4];
        private float[] _PreviousRightTrigger = new float[4];
        private bool[] _PreviousIsConnected = new bool[4];
        private KeybeardState _prevkbState = new KeybeardState();
        private MouserState _prevmouseState = new MouserState();

        #endregion

        /// <summary>
        /// UserInputService constructor.
        /// </summary>
        /// <param name="game">A reference to a Game object.</param>
        public UserInputService(Game game)
            : base(game)
        {
        }

        /// <summary>
        /// Updates the input state of the keyboard, mouse and gamepad's.
        /// </summary>
        /// <param name="gameTime">Time elapsed since the last call to Update(Microsoft.Xna.Framework.GameTime)</param>
        public override void Update(GameTime gameTime)
        {
            for (int idx = 0; idx <= 3; idx++)
            {
                GamePadState state;
                state = GamePad.GetState((PlayerIndex)idx, GamePadDeadZone.Circular);

                UpdateState(idx, state);
            }

            _prevkbState = (KeybeardState)_kbState.Clone();
            _kbState = new KeybeardState(Keyboard.GetState().GetPressedKeys());

            _prevmouseState = _mouseState;
            _mouseState = this.ConvertMouseState(Mouse.GetState());

            base.Update(gameTime);
        }

        ///<summary>
        /// Gets an array of clicked keyboard keys.
        ///</summary>
        ///<returns>Returns the keys that were pressed then released.</returns>
        public Keys[] GetClickedKeys()
        {
            Keys[] released = _kbState.GetReleasedKeys();
            List<Keys> items = new List<Keys>();
            foreach (Keys k in released)
            {
                if (_prevkbState.IsKeyDown(k)) items.Add(k);
            }

            return items.ToArray();
        }

        private MouserState ConvertMouseState(MouseState state)
        {
            MouserState ms;
            ms.LeftButton = state.LeftButton;
            ms.RightButton = state.RightButton;
            ms.MiddleButton = state.MiddleButton;
            ms.XButton1 = state.XButton1;
            ms.XButton2 = state.XButton2;
            ms.X = state.X;
            ms.Y = state.Y;
            ms.ScrollWheelValue = state.ScrollWheelValue;
            return ms;
        }

        private void UpdateState(int idx, GamePadState state)
        {
            _PreviousA[idx] = _A[idx];
            _PreviousB[idx] = _B[idx];
            _PreviousX[idx] = _X[idx];
            _PreviousY[idx] = _Y[idx];
            _PreviousBack[idx] = _Back[idx];
            _PreviousStart[idx] = _Start[idx];
            _PreviousLeftSholder[idx] = _LeftSholder[idx];
            _PreviousLeftStick[idx] = _LeftStick[idx];
            _PreviousRightSholder[idx] = _RightSholder[idx];
            _PreviousRightStick[idx] = _RightStick[idx];
            _PreviousLeft[idx] = _Left[idx];
            _PreviousRight[idx] = _Right[idx];
            _PreviousUp[idx] = _Up[idx];
            _PreviousDown[idx] = _Down[idx];
            _PreviousLeftThumbStick[idx] = _LeftThumbStick[idx];
            _PreviousRightThumbStick[idx] = _RightThumbStick[idx];
            _PreviousLeftTrigger[idx] = _LeftTrigger[idx];
            _PreviousRightTrigger[idx] = _RightTrigger[idx];
            _PreviousIsConnected[idx] = _IsConnected[idx];

            _A[idx] = state.Buttons.A;
            _B[idx] = state.Buttons.B;
            _X[idx] = state.Buttons.X;
            _Y[idx] = state.Buttons.Y;
            _Back[idx] = state.Buttons.Back;
            _Start[idx] = state.Buttons.Start;
            _LeftSholder[idx] = state.Buttons.LeftShoulder;
            _LeftStick[idx] = state.Buttons.LeftStick;
            _RightSholder[idx] = state.Buttons.RightShoulder;
            _RightStick[idx] = state.Buttons.RightStick;
            _Left[idx] = state.DPad.Left;
            _Right[idx] = state.DPad.Right;
            _Up[idx] = state.DPad.Up;
            _Down[idx] = state.DPad.Down;
            _LeftThumbStick[idx] = state.ThumbSticks.Left;
            _RightThumbStick[idx] = state.ThumbSticks.Right;
            _LeftTrigger[idx] = state.Triggers.Left;
            _RightTrigger[idx] = state.Triggers.Right;
            _IsConnected[idx] = state.IsConnected;
        }


        #region IUserInputService Members

        ///<summary>
        /// The current state of the gamepad's A button.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns the current state of the gamepad's A button.</returns>
        public ButtonState A(PlayerIndex index)
        {
            return _A[(int)index];
        }

        ///<summary>
        /// The current state of the gamepad's B button.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns the current state of the gamepad's B button.</returns>
        public ButtonState B(PlayerIndex index)
        {
            return _B[(int)index];
        }

        ///<summary>
        /// The current state of the gamepad's X button.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns the current state of the gamepad's X button.</returns>
        public ButtonState X(PlayerIndex index)
        {
            return _X[(int)index];
        }

        ///<summary>
        /// The current state of the gamepad's Y button.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns the current state of the gamepad's Y button.</returns>
        public ButtonState Y(PlayerIndex index)
        {
            return _Y[(int)index];
        }

        ///<summary>
        /// The current state of the gamepad's Back button.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns the current state of the gamepad's Back button.</returns>
        public ButtonState Back(PlayerIndex index)
        {
            return _Back[(int)index];
        }

        ///<summary>
        /// The current state of the gamepad's Start button.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns the current state of the gamepad's Start button.</returns>
        public ButtonState Start(PlayerIndex index)
        {
            return _Start[(int)index];
        }

        ///<summary>
        /// The current state of the gamepad's left sholder button.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns the current state of the gamepad's left joystick button.</returns>
        public ButtonState LeftSholder(PlayerIndex index)
        {
            return _LeftSholder[(int)index];
        }

        ///<summary>
        /// The current state of the gamepad's left joystick button.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns the current state of the gamepad's left joystick button.</returns>
        public ButtonState LeftStick(PlayerIndex index)
        {
            return _LeftStick[(int)index];
        }

        ///<summary>
        /// The current state of the gamepad's right sholder button.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns the current state of the gamepad's right sholder button.</returns>
        public ButtonState RightSholder(PlayerIndex index)
        {
            return _RightSholder[(int)index];
        }

        ///<summary>
        /// The current state of the gamepad's right joystick button.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns the current state of the gamepad's right joystick button.</returns>
        public ButtonState RightStick(PlayerIndex index)
        {
            return _RightStick[(int)index];
        }

        ///<summary>
        /// The current state of the gamepad's left DPad button.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns the current state of the gamepad's left DPad button.</returns>
        public ButtonState Left(PlayerIndex index)
        {
            return _Left[(int)index];
        }

        ///<summary>
        /// The current state of the gamepad's right DPad button.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns the current state of the gamepad's right DPad button.</returns>
        public ButtonState Right(PlayerIndex index)
        {
            return _Right[(int)index];
        }

        ///<summary>
        /// The current state of the gamepad's up DPad button.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns the current state of the gamepad's up DPad button.</returns>
        public ButtonState Up(PlayerIndex index)
        {
            return _Up[(int)index];
        }

        ///<summary>
        /// The current state of the gamepad's down DPad button.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns the current state of the gamepad's down DPad button.</returns>
        public ButtonState Down(PlayerIndex index)
        {
            return _Down[(int)index];
        }

        ///<summary>
        /// The current state of the gamepad's left thumb stick.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns the current state of the gamepad's left thumb stick.</returns>
        public Vector2 LeftThumbStick(PlayerIndex index)
        {
            return _LeftThumbStick[(int)index];
        }

        ///<summary>
        /// The current state of the gamepad's right thumb stick.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns the current state of the gamepad's right thumb stick.</returns>
        public Vector2 RightThumbStick(PlayerIndex index)
        {
            return _RightThumbStick[(int)index];
        }

        ///<summary>
        /// The current state of the gamepad's left trigger.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns the current state of the gamepad's left trigger.</returns>
        public float LeftTrigger(PlayerIndex index)
        {
            return _LeftTrigger[(int)index];
        }

        ///<summary>
        /// The current state of the gamepad's right trigger.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns the current state of the gamepad's right trigger.</returns>
        public float RightTrigger(PlayerIndex index)
        {
            return _RightTrigger[(int)index];
        }

        ///<summary>
        /// The current state of the gamepad's connection status.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns true if the game pad is connected.</returns>
        public bool IsConnected(PlayerIndex index)
        {
            return _IsConnected[(int)index];
        }

        /// <summary>
        /// Determin if the gamepad's left thumb stick has changed.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns true if the state of the gamepad's left thumb stick changed.</returns>
        public bool LeftThumbStickChanged(PlayerIndex index)
        {
            int i = (int)index;
            return _PreviousLeftThumbStick[i] != _LeftThumbStick[i];
        }

        /// <summary>
        /// Determin if the gamepad's right thumb stick has changed.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns true if the state of the gamepad's right thumb stick changed.</returns>
        public bool RightThumbStickChanged(PlayerIndex index)
        {
            int i = (int)index;
            return _PreviousRightThumbStick[i] != _RightThumbStick[i];
        }

        /// <summary>
        /// Determin if the state of the gamepad's left trigger has changed.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns true if the gamepad's left trigger was changed.</returns>
        public bool LeftTriggerChanged(PlayerIndex index)
        {
            int i = (int)index;
            return _PreviousLeftTrigger[i] != _LeftTrigger[i];
        }

        /// <summary>
        /// Determin if the state of the gamepad's right trigger has changed.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns true if the gamepad's right trigger was changed.</returns>
        public bool RightTriggerChanged(PlayerIndex index)
        {
            int i = (int)index;
            return _PreviousRightTrigger[i] != _RightTrigger[i];
        }

        ///<summary>
        /// Sets the status for the A button.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<param name="state">The specified state to set the button as.</param>
        public void A(PlayerIndex index, ButtonState state)
        {
            int i = (int)index;
            _PreviousA[i] = _A[i];
            _A[i] = state;
        }

        ///<summary>
        /// Sets the status for the B button.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<param name="state">The specified state to set the button as.</param>
        public void B(PlayerIndex index, ButtonState state)
        {
            int i = (int)index;
            _PreviousB[i] = _B[i];
            _B[i] = state;
        }

        ///<summary>
        /// Sets the status for the X button.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<param name="state">The specified state to set the button as.</param>
        public void X(PlayerIndex index, ButtonState state)
        {
            int i = (int)index;
            _PreviousX[i] = _X[i];
            _X[i] = state;
        }

        ///<summary>
        /// Sets the status for the Y button.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<param name="state">The specified state to set the button as.</param>
        public void Y(PlayerIndex index, ButtonState state)
        {
            int i = (int)index;
            _PreviousY[i] = _Y[i];
            _Y[i] = state;
        }

        ///<summary>
        /// Sets the status for the back button.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<param name="state">The specified state to set the button as.</param>
        public void Back(PlayerIndex index, ButtonState state)
        {
            int i = (int)index;
            _PreviousBack[i] = _Back[i];
            _Back[i] = state;
        }

        ///<summary>
        /// Sets the status for the start button.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<param name="state">The specified state to set the button as.</param>
        public void Start(PlayerIndex index, ButtonState state)
        {
            int i = (int)index;
            _PreviousStart[i] = _Start[i];
            _Start[i] = state;
        }

        ///<summary>
        /// Sets the status for the left holder button.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<param name="state">The specified state to set the button as.</param>
        public void LeftSholder(PlayerIndex index, ButtonState state)
        {
            int i = (int)index;
            _PreviousLeftSholder[i] = _LeftSholder[i];
            _LeftSholder[i] = state;
        }

        ///<summary>
        /// Sets the status for the left joystick button.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<param name="state">The specified state to set the button as.</param>
        public void LeftStick(PlayerIndex index, ButtonState state)
        {
            int i = (int)index;
            _PreviousLeftStick[i] = _LeftStick[i];
            _LeftStick[i] = state;
        }

        ///<summary>
        /// Sets the status for the right sholder button.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<param name="state">The specified state to set the button as.</param>
        public void RightSholder(PlayerIndex index, ButtonState state)
        {
            int i = (int)index;
            _PreviousRightSholder[i] = _RightSholder[i];
            _RightSholder[i] = state;
        }

        ///<summary>
        /// Sets the status for the right joystick button.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<param name="state">The specified state to set the button as.</param>
        public void RightStick(PlayerIndex index, ButtonState state)
        {
            int i = (int)index;
            _PreviousRightStick[i] = _RightStick[i];
            _RightStick[i] = state;
        }

        ///<summary>
        /// Sets the status for the DPad left button.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<param name="state">The specified state to set the button as.</param>
        public void Left(PlayerIndex index, ButtonState state)
        {
            int i = (int)index;
            _PreviousLeft[i] = _Left[i];
            _Left[i] = state;
        }

        ///<summary>
        /// Sets the status for the DPad right button.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<param name="state">The specified state to set the button as.</param>
        public void Right(PlayerIndex index, ButtonState state)
        {
            int i = (int)index;
            _PreviousRight[i] = _Right[i];
            _Right[i] = state;
        }

        ///<summary>
        /// Sets the status for the DPad up button.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<param name="state">The specified state to set the button as.</param>
        public void Up(PlayerIndex index, ButtonState state)
        {
            int i = (int)index;
            _PreviousUp[i] = _Up[i];
            _Up[i] = state;
        }

        ///<summary>
        /// Sets the status for the DPad down button.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<param name="state">The specified state to set the button as.</param>
        public void Down(PlayerIndex index, ButtonState state)
        {
            int i = (int)index;
            _PreviousDown[i] = _Down[i];
            _Down[i] = state;
        }

        ///<summary>
        /// Sets the status for the left thumbstick.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<param name="state">The specified state to set the thumb stick as.</param>
        public void LeftThumbStick(PlayerIndex index, Vector2 value)
        {
            int i = (int)index;
            _PreviousLeftThumbStick[i] = _LeftThumbStick[i];
            _LeftThumbStick[i] = value;
        }

        ///<summary>
        /// Sets the status for the right thumbstick.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<param name="state">The specified state to set the thumb stick as.</param>
        public void RightThumbStick(PlayerIndex index, Vector2 value)
        {
            int i = (int)index;
            _PreviousRightThumbStick[i] = _RightThumbStick[i];
            _RightThumbStick[i] = value;
        }

        ///<summary>
        /// Sets the status for the left trigger.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<param name="state">The specified state to set the trigger as.</param>
        public void LeftTrigger(PlayerIndex index, float value)
        {
            int i = (int)index;
            _PreviousLeftTrigger[i] = _LeftTrigger[i];
            _LeftTrigger[i] = value;
        }

        ///<summary>
        /// Sets the status for the right trigger.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<param name="state">The specified state to set the trigger as.</param>
        public void RightTrigger(PlayerIndex index, float value)
        {
            int i = (int)index;
            _PreviousRightTrigger[i] = _RightTrigger[i];
            _RightTrigger[i] = value;
        }

        ///<summary>
        /// Sets the connection status for the gamepad.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<param name="state">The specified state to set the connection as.</param>
        public void IsConnected(PlayerIndex index, bool value)
        {
            int i = (int)index;
            _PreviousIsConnected[i] = _IsConnected[i];
            _IsConnected[i] = value;
        }

        ///<summary>
        /// Gets the current state of a keyboard key.
        ///</summary>
        ///<param name="key">The key to be checked.</param>
        ///<returns>Returns Down if the key is down or Up if the key is released.</returns>
        public KeyState GetKeyState(Keys key)
        {
            return _kbState.keys[key];
        }

        ///<summary>
        /// Sets the current state of a keyboard key.
        ///</summary>
        ///<param name="key">The key to be set.</param>
        ///<param name="state">The state to set the key as.</param>
        public void SetKeyState(Keys key, KeyState state)
        {
            _kbState.keys[key] = state;
        }

        ///<summary>
        /// Gets the previous state of a keyboard key.
        ///</summary>
        ///<param name="key">The key to be checked.</param>
        ///<returns>Returns Down if the key is down or Up if the key is released.</returns>
        public KeyState GetPreviuosKeyState(Keys key)
        {
            return _prevkbState.keys[key];
        }

        ///<summary>
        /// Sets the previous state of a keyboard key.
        ///</summary>
        ///<param name="key">The key to be set.</param>
        ///<param name="state">The state to set the key as.</param>
        public void SetPreviuosKeyState(Keys key, KeyState state)
        {
            _prevkbState.keys[key] = state;
        }

        ///<summary>
        /// The previous state of the gamepad's A button.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns the previous state of the gamepad's A button.</returns>
        public ButtonState PreviousA(PlayerIndex index)
        {
            return _PreviousA[(int)index];
        }

        ///<summary>
        /// The previous state of the gamepad's B button.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns the previous state of the gamepad's B button.</returns>
        public ButtonState PreviousB(PlayerIndex index)
        {
            return _PreviousB[(int)index];
        }

        ///<summary>
        /// The previous state of the gamepad's X button.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns the previous state of the gamepad's X button.</returns>
        public ButtonState PreviousX(PlayerIndex index)
        {
            return _PreviousX[(int)index];
        }

        ///<summary>
        /// The previous state of the gamepad's Y button.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns the previous state of the gamepad's Y button.</returns>
        public ButtonState PreviousY(PlayerIndex index)
        {
            return _PreviousY[(int)index];
        }

        ///<summary>
        /// The previous state of the gamepad's Back button.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns the previous state of the gamepad's Back button.</returns>
        public ButtonState PreviousBack(PlayerIndex index)
        {
            return _PreviousBack[(int)index];
        }

        ///<summary>
        /// The previous state of the gamepad's Start button.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns the previous state of the gamepad's Start button.</returns>
        public ButtonState PreviousStart(PlayerIndex index)
        {
            return _PreviousStart[(int)index];
        }

        ///<summary>
        /// The previous state of the gamepad's left sholder button.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns the previous state of the gamepad's left joystick button.</returns>
        public ButtonState PreviousLeftSholder(PlayerIndex index)
        {
            return _PreviousLeftSholder[(int)index];
        }

        ///<summary>
        /// The previous state of the gamepad's left joystick button.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns the previous state of the gamepad's left joystick button.</returns>
        public ButtonState PreviousLeftStick(PlayerIndex index)
        {
            return _PreviousLeftStick[(int)index];
        }

        ///<summary>
        /// The previous state of the gamepad's right sholder button.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns the previous state of the gamepad's right sholder button.</returns>
        public ButtonState PreviousRightSholder(PlayerIndex index)
        {
            return _PreviousRightSholder[(int)index];
        }

        ///<summary>
        /// The previous state of the gamepad's right joystick button.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns the previous state of the gamepad's right joystick button.</returns>
        public ButtonState PreviousRightStick(PlayerIndex index)
        {
            return _PreviousRightStick[(int)index];
        }

        ///<summary>
        /// The previous state of the gamepad's left DPad button.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns the previous state of the gamepad's left DPad button.</returns>
        public ButtonState PreviousLeft(PlayerIndex index)
        {
            return _PreviousLeft[(int)index];
        }

        ///<summary>
        /// The previous state of the gamepad's right DPad button.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns the previous state of the gamepad's right DPad button.</returns>
        public ButtonState PreviousRight(PlayerIndex index)
        {
            return _PreviousRight[(int)index];
        }

        ///<summary>
        /// The previous state of the gamepad's up DPad button.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns the previous state of the gamepad's up DPad button.</returns>
        public ButtonState PreviousUp(PlayerIndex index)
        {
            return _PreviousUp[(int)index];
        }

        ///<summary>
        /// The previous state of the gamepad's down DPad button.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns the previous state of the gamepad's down DPad button.</returns>
        public ButtonState PreviousDown(PlayerIndex index)
        {
            return _PreviousDown[(int)index];
        }

        ///<summary>
        /// The previous state of the gamepad's left thumb stick.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns the previous state of the gamepad's left thumb stick.</returns>
        public Vector2 PreviousLeftThumbStick(PlayerIndex index)
        {
            return _PreviousLeftThumbStick[(int)index];
        }

        ///<summary>
        /// The previous state of the gamepad's right thumb stick.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns the previous state of the gamepad's right thumb stick.</returns>
        public Vector2 PreviousRightThumbStick(PlayerIndex index)
        {
            return _PreviousRightThumbStick[(int)index];
        }

        ///<summary>
        /// The previous state of the gamepad's left trigger.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns the previous state of the gamepad's left trigger.</returns>
        public float PreviousLeftTrigger(PlayerIndex index)
        {
            return _PreviousLeftTrigger[(int)index];
        }

        ///<summary>
        /// The previous state of the gamepad's right trigger.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns the previous state of the gamepad's right trigger.</returns>
        public float PreviousRightTrigger(PlayerIndex index)
        {
            return _PreviousRightTrigger[(int)index];
        }

        ///<summary>
        /// The previous state of the gamepad's connection status.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns true if the game pad was connected.</returns>
        public bool WasConnected(PlayerIndex index)
        {
            return _PreviousIsConnected[(int)index];
        }

        ///<summary>
        /// Gets the state of a keyboard key.
        ///</summary>
        ///<param name="key">The key to check.</param>
        ///<returns>Will return true if the key is pressed.</returns>
        public bool IsKeyDown(Keys key)
        {
            return _kbState.IsKeyDown(key);
        }

        ///<summary>
        /// Gets the state of a keyboard key.
        ///</summary>
        ///<param name="key">The key to check.</param>
        ///<returns>Will return true if the key is released.</returns>
        public bool IsKeyUp(Keys key)
        {
            return _kbState.IsKeyUp(key);
        }

        ///<summary>
        /// Gets the previous state of a keyboard key.
        ///</summary>
        ///<param name="key">The key to check.</param>
        ///<returns>Will return true if the key was pressed.</returns>
        public bool WasKeyDown(Keys key)
        {
            return _prevkbState.keys[key] == KeyState.Down;
        }

        ///<summary>
        /// Gets the previous state of a keyboard key.
        ///</summary>
        ///<param name="key">The key to check.</param>
        ///<returns>Will return true if the key was released.</returns>
        public bool WasKeyUp(Keys key)
        {
            return _prevkbState.keys[key] == KeyState.Up;
        }

        ///<summary>
        /// Gets an array of current keys that are being pressed on the keyboard.
        ///</summary>
        ///<returns>Returns an array of pressed keys on the keyboard.</returns>
        public Keys[] PressedKeys()
        {
            return _kbState.GetPressedKeys();
        }

        ///<summary>
        /// Gets an array of previous keys that were being pressed on the keyboard.
        ///</summary>
        ///<returns>Returns an array of pressed keys on the keyboard.</returns>
        public Keys[] PreviouslyPressedKeys()
        {
            return _prevkbState.GetPressedKeys();
        }

        ///<summary>
        /// Gets an array of current keys that are not being pressed on the keyboard.
        ///</summary>
        ///<returns>Returns an array of released keys on the keyboard.</returns>
        public Keys[] ReleasedKeys()
        {
            return _kbState.GetReleasedKeys();
        }

        ///<summary>
        /// Gets an array of previous keys that were not being pressed on the keyboard.
        ///</summary>
        ///<returns>Returns an array of released keys on the keyboard.</returns>
        public Keys[] PreviouslyReleasedKeys()
        {
            return _prevkbState.GetReleasedKeys();
        }

        /// <summary>
        /// Determins if a keyboard key was clicked.
        /// </summary>
        /// <param name="key">The keyboard key to check.</param>
        /// <returns>Returns true if the key was pressed then released.</returns>
        public bool KeyClick(Keys key)
        {
            return _prevkbState.IsKeyDown(key) & _kbState.IsKeyUp(key);
        }

#if Win32
        /// <summary>
        /// Gets/Sets the current state of the left mouse button.
        /// </summary>
        public bool IsLeftMouseButtonDown
        {
            get { return _mouseState.LeftButton == ButtonState.Pressed; }
            set { _mouseState.LeftButton = value ? ButtonState.Pressed : ButtonState.Released; }
        }

        /// <summary>
        /// Gets/Sets the current state of the right mouse button.
        /// </summary>
        public bool IsRightMouseButtonDown
        {
            get { return _mouseState.RightButton == ButtonState.Pressed; }
            set { _mouseState.RightButton = value ? ButtonState.Pressed : ButtonState.Released; }
        }

        /// <summary>
        /// Gets/Sets the current state of the middle mouse button.
        /// </summary>
        public bool IsMiddleMouseButtonDown
        {
            get { return _mouseState.MiddleButton == ButtonState.Pressed; }
            set { _mouseState.MiddleButton = value ? ButtonState.Pressed : ButtonState.Released; }
        }

        /// <summary>
        /// Gets/Sets the current state of the XButton1 mouse button.
        /// </summary>
        public bool IsXButton1MouseButtonDown
        {
            get { return _mouseState.XButton1 == ButtonState.Pressed; }
            set { _mouseState.XButton1 = value ? ButtonState.Pressed : ButtonState.Released; }
        }

        /// <summary>
        /// Gets/Sets the current state of the XButton2 mouse button.
        /// </summary>
        public bool IsXButton2MouseButtonDown
        {
            get { return _mouseState.XButton2 == ButtonState.Pressed; }
            set { _mouseState.XButton2 = value ? ButtonState.Pressed : ButtonState.Released; }
        }

        /// <summary>
        /// Gets/Sets the current state of the left mouse button.
        /// </summary>
        public bool IsLeftMouseButtonUp
        {
            get { return _mouseState.LeftButton == ButtonState.Released; }
            set { _mouseState.LeftButton = value ? ButtonState.Released : ButtonState.Pressed; }
        }

        /// <summary>
        /// Gets/Sets the current state of the right mouse button.
        /// </summary>
        public bool IsRightMouseButtonUp
        {
            get { return _mouseState.RightButton == ButtonState.Released; }
            set { _mouseState.RightButton = value ? ButtonState.Released : ButtonState.Pressed; }
        }

        /// <summary>
        /// Gets/Sets the current state of the middle mouse button.
        /// </summary>
        public bool IsMiddleMouseButtonUp
        {
            get { return _mouseState.MiddleButton == ButtonState.Released; }
            set { _mouseState.MiddleButton = value ? ButtonState.Released : ButtonState.Pressed; }
        }

        /// <summary>
        /// Gets/Sets the current state of the XButton1 mouse button.
        /// </summary>
        public bool IsXButton1MouseButtonUp
        {
            get { return _mouseState.XButton1 == ButtonState.Released; }
            set { _mouseState.XButton1 = value ? ButtonState.Released : ButtonState.Pressed; }
        }

        /// <summary>
        /// Gets/Sets the current state of the XButton2 mouse button.
        /// </summary>
        public bool IsXButton2MouseButtonUp
        {
            get { return _mouseState.XButton2 == ButtonState.Released; }
            set { _mouseState.XButton2 = value ? ButtonState.Released : ButtonState.Pressed; }
        }

        /// <summary>
        /// Gets/Sets the current x location of the mouse.
        /// </summary>
        public int MouseX
        {
            get { return _mouseState.X; }
            set { _mouseState.X = value; }
        }

        /// <summary>
        /// Gets/Sets the current y location of the mouse.
        /// </summary>
        public int MouseY
        {
            get { return _mouseState.Y; }
            set { _mouseState.Y = value; }
        }

        /// <summary>
        /// Gets/Sets the current position of the mouse.
        /// </summary>
        public Vector2 MousePosition
        {
            get { return new Vector2(_mouseState.X, _mouseState.Y); }
            set
            {
                _mouseState.X = (int)value.X;
                _mouseState.Y = (int)value.Y;
            }
        }

        /// <summary>
        /// Gets/Sets the current scroll wheel value of the mouse.
        /// </summary>
        public int MouseScrollWheel
        {
            get { return _mouseState.ScrollWheelValue; }
            set { _mouseState.ScrollWheelValue = value; }
        }

        /// <summary>
        /// Gets/Sets the previous state of the left mouse button.
        /// </summary>
        public bool WasLeftMouseButtonDown
        {
            get { return _prevmouseState.LeftButton == ButtonState.Pressed; }
            set { _prevmouseState.LeftButton = value ? ButtonState.Pressed : ButtonState.Released; }
        }

        /// <summary>
        /// Gets/Sets the previous state of the right mouse button.
        /// </summary>
        public bool WasRightMouseButtonDown
        {
            get { return _prevmouseState.RightButton == ButtonState.Pressed; }
            set { _prevmouseState.LeftButton = value ? ButtonState.Pressed : ButtonState.Released; }
        }

        /// <summary>
        /// Gets/Sets the previous state of the middle mouse button.
        /// </summary>
        public bool WasMiddleMouseButtonDown
        {
            get { return _prevmouseState.MiddleButton == ButtonState.Pressed; }
            set { _prevmouseState.LeftButton = value ? ButtonState.Pressed : ButtonState.Released; }
        }

        /// <summary>
        /// Gets/Sets the previous state of the XButton1 mouse button.
        /// </summary>
        public bool WasXButton1MouseButtonDown
        {
            get { return _prevmouseState.XButton1 == ButtonState.Pressed; }
            set { _prevmouseState.LeftButton = value ? ButtonState.Pressed : ButtonState.Released; }
        }

        /// <summary>
        /// Gets/Sets the previous state of the XButton2 mouse button.
        /// </summary>
        public bool WasXButton2MouseButtonDown
        {
            get { return _prevmouseState.XButton2 == ButtonState.Pressed; }
            set { _prevmouseState.LeftButton = value ? ButtonState.Pressed : ButtonState.Released; }
        }

        /// <summary>
        /// Gets/Sets the previous state of the left mouse button.
        /// </summary>
        public bool WasLeftMouseButtonUp
        {
            get { return _prevmouseState.LeftButton == ButtonState.Released; }
            set { _prevmouseState.LeftButton = value ? ButtonState.Released : ButtonState.Pressed; }
        }

        /// <summary>
        /// Gets/Sets the previous state of the right mouse button.
        /// </summary>
        public bool WasRightMouseButtonUp
        {
            get { return _prevmouseState.RightButton == ButtonState.Released; }
            set { _prevmouseState.LeftButton = value ? ButtonState.Released : ButtonState.Pressed; }
        }

        /// <summary>
        /// Gets/Sets the previous state of the middle mouse button.
        /// </summary>
        public bool WasMiddleMouseButtonUp
        {
            get { return _prevmouseState.MiddleButton == ButtonState.Released; }
            set { _prevmouseState.LeftButton = value ? ButtonState.Released : ButtonState.Pressed; }
        }

        /// <summary>
        /// Gets/Sets the previous state of the XButton1 mouse button.
        /// </summary>
        public bool WasXButton1MouseButtonUp
        {
            get { return _prevmouseState.XButton1 == ButtonState.Released; }
            set { _prevmouseState.LeftButton = value ? ButtonState.Released : ButtonState.Pressed; }
        }

        /// <summary>
        /// Gets/Sets the previous state of the XButton2 mouse button.
        /// </summary>
        public bool WasXButton2MouseButtonUp
        {
            get { return _prevmouseState.XButton2 == ButtonState.Released; }
            set { _prevmouseState.LeftButton = value ? ButtonState.Released : ButtonState.Pressed; }
        }

        /// <summary>
        /// Gets/Sets the previous x location of the mouse.
        /// </summary>
        public int PreviousMouseX
        {
            get { return _prevmouseState.X; }
            set { _prevmouseState.X = value; }
        }

        /// <summary>
        /// Gets/Sets the previous y location of the mouse.
        /// </summary>
        public int PreviousMouseY
        {
            get { return _prevmouseState.Y; }
            set { _prevmouseState.Y = value; }
        }

        /// <summary>
        /// Gets/Sets the previous position of the mouse.
        /// </summary>
        public Vector2 PreviousMousePosition
        {
            get { return new Vector2(_prevmouseState.X, _prevmouseState.Y); }
            set
            {
                _prevmouseState.X = (int)value.X;
                _prevmouseState.Y = (int)value.Y;
            }
        }

        /// <summary>
        /// Gets/Sets the previous scroll wheel value of the mouse.
        /// </summary>
        public int PreviousMouseScrollWheel
        {
            get { return _prevmouseState.ScrollWheelValue; }
            set { _prevmouseState.ScrollWheelValue = value; }
        }

        /// <summary>
        /// Determins if the left mouse button button was clicked.
        /// </summary>
        /// <returns>Returns true if the button was pressed then released.</returns>
        public bool LeftMouseButtonClicked
        {
            get { return _prevmouseState.LeftButton == ButtonState.Pressed && _mouseState.LeftButton == ButtonState.Released; }
        }

        /// <summary>
        /// Determins if the right mouse button button was clicked.
        /// </summary>
        /// <returns>Returns true if the button was pressed then released.</returns>
        public bool RightMouseButtonClicked
        {
            get { return _prevmouseState.RightButton == ButtonState.Pressed && _mouseState.RightButton == ButtonState.Released; }
        }

        /// <summary>
        /// Determins if the middle mouse button button was clicked.
        /// </summary>
        /// <returns>Returns true if the button was pressed then released.</returns>
        public bool MiddleMouseButtonClicked
        {
            get { return _prevmouseState.MiddleButton == ButtonState.Pressed && _mouseState.MiddleButton == ButtonState.Released; }
        }

        /// <summary>
        /// Determins if the XButton1 mouse button button was clicked.
        /// </summary>
        /// <returns>Returns true if the button was pressed then released.</returns>
        public bool XButton1MouseButtonClicked
        {
            get { return _prevmouseState.XButton1 == ButtonState.Pressed && _mouseState.XButton1 == ButtonState.Released; }
        }

        /// <summary>
        /// Determins if the Xbutton2 mouse button button was clicked.
        /// </summary>
        /// <returns>Returns true if the button was pressed then released.</returns>
        public bool XButton2MouseButtonClicked
        {
            get { return _prevmouseState.XButton2 == ButtonState.Pressed && _mouseState.XButton2 == ButtonState.Released; }
        }

        /// <summary>
        /// Determins if the X position of the mouse has changed.
        /// </summary>
        /// <returns>Returns true if the mouse has changed position.</returns>
        public bool MouseXChanged
        {
            get { return _prevmouseState.X != _mouseState.X; }
        }

        /// <summary>
        /// Determins if the Y position of the mouse has changed.
        /// </summary>
        /// <returns>Returns true if the mouse has changed position.</returns>
        public bool MouseYChanged
        {
            get { return _prevmouseState.Y != _mouseState.Y; }
        }

        /// <summary>
        /// Determins if the scroll wheel position of the mouse has changed.
        /// </summary>
        /// <returns>Returns true if the mouse scroll wheel has changed position.</returns>
        public bool MouseScrollWheelChanged
        {
            get { return _prevmouseState.ScrollWheelValue != _mouseState.ScrollWheelValue; }
        }
#endif

        ///<summary>
        ///     Determins if the A button was clicked.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns true if the button was pressed then released.</returns>
        public bool AClick(PlayerIndex index)
        {
            int idx = (int)index;
            return _PreviousA[idx] == ButtonState.Pressed && _A[idx] == ButtonState.Released;
        }

        ///<summary>
        /// Determins if the B button was clicked.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns true if the button was pressed then released.</returns>
        public bool BClick(PlayerIndex index)
        {
            int idx = (int)index;
            return _PreviousB[idx] == ButtonState.Pressed && _B[idx] == ButtonState.Released;
        }

        ///<summary>
        ///     Determins if the X button was clicked.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns true if the button was pressed then released.</returns>
        public bool XClick(PlayerIndex index)
        {
            int idx = (int)index;
            return _PreviousX[idx] == ButtonState.Pressed && _X[idx] == ButtonState.Released;
        }

        ///<summary>
        /// Determins if the Y button was clicked.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns true if the button was pressed then released.</returns>
        public bool YClick(PlayerIndex index)
        {
            int idx = (int)index;
            return _PreviousY[idx] == ButtonState.Pressed && _Y[idx] == ButtonState.Released;
        }

        ///<summary>
        ///    Determins if the Back button was clicked.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns true if the button was pressed then released.</returns>
        public bool BackClick(PlayerIndex index)
        {
            int idx = (int)index;
            return _PreviousBack[idx] == ButtonState.Pressed && _Back[idx] == ButtonState.Released;
        }

        ///<summary>
        /// Determins if the Start button was clicked.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns true if the button was pressed then released.</returns>
        public bool StartClick(PlayerIndex index)
        {
            int idx = (int)index;
            return _PreviousStart[idx] == ButtonState.Pressed && _Start[idx] == ButtonState.Released;
        }

        ///<summary>
        ///      Determins if the left sholder button was clicked.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns true if the button was pressed then released.</returns>
        public bool LeftSholderClick(PlayerIndex index)
        {
            int idx = (int)index;
            return _PreviousLeftSholder[idx] == ButtonState.Pressed && _LeftSholder[idx] == ButtonState.Released;
        }

        ///<summary>
        /// Determins if the left stick button was clicked.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns true if the button was pressed then released.</returns>
        public bool LeftStickClick(PlayerIndex index)
        {
            int idx = (int)index;
            return _PreviousLeftStick[idx] == ButtonState.Pressed && _LeftStick[idx] == ButtonState.Released;
        }

        ///<summary>
        ///      Determins if the right sholder button was clicked.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns true if the button was pressed then released.</returns>
        public bool RightSholderClick(PlayerIndex index)
        {
            int idx = (int)index;
            return _PreviousRightSholder[idx] == ButtonState.Pressed && _RightSholder[idx] == ButtonState.Released;
        }

        ///<summary>
        /// Determins if the right stick button was clicked.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns true if the button was pressed then released.</returns>
        public bool RightStickClick(PlayerIndex index)
        {
            int idx = (int)index;
            return _PreviousRightStick[idx] == ButtonState.Pressed && _RightStick[idx] == ButtonState.Released;
        }

        ///<summary>
        ///    Determins if the left DPad button was clicked.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns true if the button was pressed then released.</returns>
        public bool LeftClick(PlayerIndex index)
        {
            int idx = (int)index;
            return _PreviousLeft[idx] == ButtonState.Pressed && _Left[idx] == ButtonState.Released;
        }

        ///<summary>
        /// Determins if the right DPad button was clicked.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns true if the button was pressed then released.</returns>
        public bool RightClick(PlayerIndex index)
        {
            int idx = (int)index;
            return _PreviousRight[idx] == ButtonState.Pressed && _Right[idx] == ButtonState.Released;
        }

        ///<summary>
        ///       Determins if the up DPad button was clicked.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns true if the button was pressed then released.</returns>
        public bool UpClick(PlayerIndex index)
        {
            int idx = (int)index;
            return _PreviousUp[idx] == ButtonState.Pressed && _Up[idx] == ButtonState.Released;
        }

        ///<summary>
        /// Determins if the down DPad button was clicked.
        ///</summary>
        ///<param name="index">The index of the player.</param>
        ///<returns>Returns true if the button was pressed then released.</returns>
        public bool DownClick(PlayerIndex index)
        {
            int idx = (int)index;
            return _PreviousDown[idx] == ButtonState.Pressed && _Down[idx] == ButtonState.Released;
        }

        #endregion
    }
}